@@ -47,8 +47,8 @@ class TCoffeeCommandline(AbstractCommandline):
                     0),
            _Switch(["-convert", "convert"], ["input"],
                     "Specify you want to perform a file conversion"),
-           _Option(["-type"], ["input"],
-                    lambda x: x in SEQ_TYPES,
+           _Option(["-type", "type"], ["input"],
+                    lambda x: x in self.SEQ_TYPES,
                     0,
                     "Specify the type of sequence being aligned",
                     0),
