@@ -1,4 +1,4 @@
-# Copyright 2008-2009 by Peter Cock.  All rights reserved.
+# Copyright 2008-2010 by Peter Cock.  All rights reserved.
 #
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -13,7 +13,7 @@ This module contains a parser for the EMBOSS pairs/simple file format, for
 example from the alignret, water and needle tools.
 """
 
-from Bio.Align.Generic import Alignment
+from Bio.Align import MultipleSeqAlignment
 from Interfaces import AlignmentIterator, SequentialAlignmentWriter
 
 class EmbossWriter(SequentialAlignmentWriter):
@@ -41,14 +41,11 @@ class EmbossWriter(SequentialAlignmentWriter):
         
     def write_alignment(self, alignment):
         """Use this to write (another) single alignment to an open file."""
-
         handle = self.handle
-        records = alignment.get_all_seqs()
-        
         handle.write("#=======================================\n")
         handle.write("#\n")
-        handle.write("# Aligned_sequences: %i\n" % len(records))
-        for i, record in enumerate(records):
+        handle.write("# Aligned_sequences: %i\n" % len(alignment))
+        for i, record in enumerate(alignment):
             handle.write("# %i: %s\n" % (i+1, record.id))
         handle.write("#\n")
         handle.write("# Length: %i\n" % alignment.get_alignment_length())
@@ -205,7 +202,7 @@ class EmbossIterator(AlignmentIterator):
             raise ValueError("Found %i records in this alignment, told to expect %i" \
                              % (len(ids), self.records_per_alignment))
 
-        alignment = Alignment(self.alphabet)
+        alignment = MultipleSeqAlignment(self.alphabet)
         for id, seq in zip(ids, seqs):
             if len(seq) != length_of_seqs:
                 #EMBOSS 2.9.0 is known to use spaces instead of minus signs
@@ -581,38 +578,38 @@ asis             311 -----------------    311
 
     alignments = list(EmbossIterator(StringIO(pair_example)))
     assert len(alignments) == 1
-    assert len(alignments[0].get_all_seqs()) == 2
-    assert [r.id for r in alignments[0].get_all_seqs()] \
+    assert len(alignments[0]) == 2
+    assert [r.id for r in alignments[0]] \
            == ["IXI_234", "IXI_235"]
     
     alignments = list(EmbossIterator(StringIO(simple_example)))
     assert len(alignments) == 1    
-    assert len(alignments[0].get_all_seqs()) == 4
-    assert [r.id for r in alignments[0].get_all_seqs()] \
+    assert len(alignments[0]) == 4
+    assert [r.id for r in alignments[0]] \
            == ["IXI_234", "IXI_235", "IXI_236", "IXI_237"]
 
     alignments = list(EmbossIterator(StringIO(pair_example + simple_example)))
     assert len(alignments) == 2    
-    assert len(alignments[0].get_all_seqs()) == 2
-    assert len(alignments[1].get_all_seqs()) == 4
-    assert [r.id for r in alignments[0].get_all_seqs()] \
+    assert len(alignments[0]) == 2
+    assert len(alignments[1]) == 4
+    assert [r.id for r in alignments[0]] \
            == ["IXI_234", "IXI_235"]
-    assert [r.id for r in alignments[1].get_all_seqs()] \
+    assert [r.id for r in alignments[1]] \
            == ["IXI_234", "IXI_235", "IXI_236", "IXI_237"]
 
     alignments = list(EmbossIterator(StringIO(pair_example2)))
     assert len(alignments) == 5
-    assert len(alignments[0].get_all_seqs()) == 2
-    assert [r.id for r in alignments[0].get_all_seqs()] \
+    assert len(alignments[0]) == 2
+    assert [r.id for r in alignments[0]] \
            == ["ref_rec", "gi|94968718|receiver"]
-    assert [r.id for r in alignments[4].get_all_seqs()] \
+    assert [r.id for r in alignments[4]] \
            == ["ref_rec", "gi|94970041|receiver"]
 
 
     alignments = list(EmbossIterator(StringIO(pair_example3)))
     assert len(alignments) == 1
-    assert len(alignments[0].get_all_seqs()) == 2
-    assert [r.id for r in alignments[0].get_all_seqs()] \
+    assert len(alignments[0]) == 2
+    assert [r.id for r in alignments[0]] \
            == ["asis","asis"]
 
     print "Done"
