@@ -1,4 +1,4 @@
-# Copyright 2006-2009 by Peter Cock.  All rights reserved.
+# Copyright 2006-2010 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -40,7 +40,7 @@ using the Bio.AlignIO.read() function:
     AP001509.1 104
     AE007476.1 104
 
-This example file is clearly using RNA, so you might want the Alignment object
+This example file is clearly using RNA, so you might want the alignment object
 (and the SeqRecord objects it holds) to reflect this, rather than simple using
 the default single letter alphabet as shown above.  You can do this with an
 optional argument to the Bio.AlignIO.read() function:
@@ -72,7 +72,7 @@ with one character for each letter in the associated sequence:
 
 Any general annotation for each row is recorded in the SeqRecord's annotations
 dictionary.  You can output this alignment in many different file formats using
-Bio.AlignIO.write(), or the Alignment object's format method:
+Bio.AlignIO.write(), or the MultipleSeqAlignment object's format method:
 
     >>> print align.format("fasta")
     >AP001509.1
@@ -132,7 +132,9 @@ secondary structure string here, are also sliced:
     -------<<<
 """
 __docformat__ = "epytext en" #not just plaintext
-from Bio.Align.Generic import Alignment
+from Bio.Seq import Seq
+from Bio.SeqRecord import SeqRecord
+from Bio.Align import MultipleSeqAlignment
 from Interfaces import AlignmentIterator, SequentialAlignmentWriter
 
 class StockholmWriter(SequentialAlignmentWriter):
@@ -159,8 +161,7 @@ class StockholmWriter(SequentialAlignmentWriter):
         together (rather than having a block of annotation followed
         by a block of aligned sequences).
         """
-        records = alignment.get_all_seqs()
-        count = len(records)
+        count = len(alignment)
         
         self._length_of_sequences = alignment.get_alignment_length()
         self._ids_written = []
@@ -175,7 +176,7 @@ class StockholmWriter(SequentialAlignmentWriter):
 
         self.handle.write("# STOCKHOLM 1.0\n")
         self.handle.write("#=GF SQ %i\n" % count)
-        for record in records:
+        for record in alignment:
             self._write_record(record)
         self.handle.write("//\n")
 
@@ -266,7 +267,7 @@ class StockholmWriter(SequentialAlignmentWriter):
                 pass
         
 class StockholmIterator(AlignmentIterator):
-    """Loads a Stockholm file from PFAM into Alignment objects.
+    """Loads a Stockholm file from PFAM into MultipleSeqAlignment objects.
 
     The file may contain multiple concatenated alignments, which are loaded
     and returned incrementally.
@@ -423,7 +424,7 @@ class StockholmIterator(AlignmentIterator):
                 raise ValueError("Found %i records in this alignment, told to expect %i" \
                                  % (len(ids), self.records_per_alignment))
 
-            alignment = Alignment(self.alphabet)
+            alignment = MultipleSeqAlignment(self.alphabet)
 
             #TODO - Introduce an annotated alignment class?
             #For now, store the annotation a new private property:
@@ -435,21 +436,20 @@ class StockholmIterator(AlignmentIterator):
                 if alignment_length != len(seq):
                     raise ValueError("Sequences have different lengths, or repeated identifier")
                 name, start, end = self._identifier_split(id)
-                alignment.add_sequence(id, seq, start=start, end=end)
-
-                record = alignment.get_all_seqs()[-1]
-
-                assert record.id == id or record.description == id
-                
-                record.id = id
-                record.name = name
-                record.description = id
-                
-                #will be overridden by _populate_meta_data if an explicit
+                record = SeqRecord(Seq(seq, self.alphabet),
+                                   id = id, name = name, description = id,
+                                   annotations = {"accession":name})
+                #Accession will be overridden by _populate_meta_data if an explicit
                 #accession is provided:
                 record.annotations["accession"]=name
 
+                if start is not None:
+                    record.annotations["start"] = start
+                if end is not None:
+                    record.annotations["end"] = end
+
                 self._populate_meta_data(id, record)
+                alignment.append(record)
             return alignment
         else:
             return None
