@@ -1,9 +1,9 @@
-# Copyright 2008-2009 by Peter Cock.  All rights reserved.
+# Copyright 2008-2010 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-"""Multiple sequence alignment input/output as Alignment objects.
+"""Multiple sequence alignment input/output as alignment objects.
 
 The Bio.AlignIO interface is deliberately very similar to Bio.SeqIO, and in
 fact the two are connected internally.  Both modules use the same set of file
@@ -20,14 +20,13 @@ Input
 =====
 For the typical special case when your file or handle contains one and only
 one alignment, use the function Bio.AlignIO.read().  This takes an input file
-handle, format string and optional number of sequences per alignment.  It will
-return a single Alignment object (or raise an exception if there isn't just
-one alignment):
+handle (or in recent versions of Biopython a filename as a string), format
+string and optional number of sequences per alignment.  It will return a single
+MultipleSeqAlignment object (or raise an exception if there isn't just one
+alignment):
 
     >>> from Bio import AlignIO
-    >>> handle = open("Phylip/interlaced.phy", "rU")
-    >>> align = AlignIO.read(handle, "phylip")
-    >>> handle.close()
+    >>> align = AlignIO.read("Phylip/interlaced.phy", "phylip")
     >>> print align
     SingleLetterAlphabet() alignment with 3 rows and 384 columns
     -----MKVILLFVLAVFTVFVSS---------------RGIPPE...I-- CYS1_DICDI
@@ -36,12 +35,12 @@ one alignment):
 
 For the general case, when the handle could contain any number of alignments,
 use the function Bio.AlignIO.parse(...) which takes the same arguments, but
-returns an iterator giving Alignment objects (typically used in a for loop).
-If you want random access to the alignments by number, turn this into a list:
+returns an iterator giving MultipleSeqAlignment objects (typically used in a
+for loop). If you want random access to the alignments by number, turn this
+into a list:
 
     >>> from Bio import AlignIO
-    >>> handle = open("Emboss/needle.txt", "rU")
-    >>> alignments = list(AlignIO.parse(handle, "emboss"))
+    >>> alignments = list(AlignIO.parse("Emboss/needle.txt", "emboss"))
     >>> print alignments[2]
     SingleLetterAlphabet() alignment with 2 rows and 120 columns
     -KILIVDDQYGIRILLNEVFNKEGYQTFQAANGLQALDIVTKER...--- ref_rec
@@ -56,19 +55,28 @@ Output
 ======
 Use the function Bio.AlignIO.write(...), which takes a complete set of
 Alignment objects (either as a list, or an iterator), an output file handle
-and of course the file format::
+(or filename in recent versions of Biopython) and of course the file format::
+
+    from Bio import AlignIO
+    alignments = ...
+    count = SeqIO.write(alignments, "example.faa", "fasta")
+
+If using a handle make sure to close it to flush the data to the disk::
 
     from Bio import AlignIO
     alignments = ...
     handle = open("example.faa", "w")
-    alignment = SeqIO.write(alignments, handle, "fasta")
+    count = SeqIO.write(alignments, handle, "fasta")
     handle.close()
 
 In general, you are expected to call this function once (with all your
 alignments) and then close the file handle.  However, for file formats
 like PHYLIP where multiple alignments are stored sequentially (with no file
 header and footer), then multiple calls to the write function should work as
-expected.
+expected when using handles.
+
+If you are using a filename, the repeated calls to the write functions will
+overwrite the existing file each time.
 
 Conversion
 ==========
@@ -125,11 +133,11 @@ __docformat__ = "epytext en" #not just plaintext
 # - MSF multiple alignment format, aka GCG, aka PileUp format (*.msf)
 #   http://www.bioperl.org/wiki/MSF_multiple_alignment_format 
 
-import os
 #from cStringIO import StringIO
 from StringIO import StringIO
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
+from Bio.Align import MultipleSeqAlignment
 from Bio.Align.Generic import Alignment
 from Bio.Alphabet import Alphabet, AlphabetEncoder, _get_base_alphabet
 
@@ -143,29 +151,32 @@ import FastaIO
 #Convention for format names is "mainname-subtype" in lower case.
 #Please use the same names as BioPerl and EMBOSS where possible.
 
-_FormatToIterator ={#"fasta" is done via Bio.SeqIO
-                    "clustal" : ClustalIO.ClustalIterator,
-                    "emboss" : EmbossIO.EmbossIterator,
-                    "fasta-m10" : FastaIO.FastaM10Iterator,
-                    "nexus" : NexusIO.NexusIterator,
-                    "phylip" : PhylipIO.PhylipIterator,
-                    "stockholm" : StockholmIO.StockholmIterator,
-                    }
+_FormatToIterator = {#"fasta" is done via Bio.SeqIO
+                     "clustal" : ClustalIO.ClustalIterator,
+                     "emboss" : EmbossIO.EmbossIterator,
+                     "fasta-m10" : FastaIO.FastaM10Iterator,
+                     "nexus" : NexusIO.NexusIterator,
+                     "phylip" : PhylipIO.PhylipIterator,
+                     "stockholm" : StockholmIO.StockholmIterator,
+                     }
 
-_FormatToWriter ={#"fasta" is done via Bio.SeqIO
-                  #"emboss" : EmbossIO.EmbossWriter, (unfinished)
-                  "nexus" : NexusIO.NexusWriter,
-                  "phylip" : PhylipIO.PhylipWriter,
-                  "stockholm" : StockholmIO.StockholmWriter,
-                  "clustal" : ClustalIO.ClustalWriter,
-                  }
+_FormatToWriter = {#"fasta" is done via Bio.SeqIO
+                   #"emboss" : EmbossIO.EmbossWriter, (unfinished)
+                   "nexus" : NexusIO.NexusWriter,
+                   "phylip" : PhylipIO.PhylipWriter,
+                   "stockholm" : StockholmIO.StockholmWriter,
+                   "clustal" : ClustalIO.ClustalWriter,
+                   }
 
 def write(alignments, handle, format):
     """Write complete set of alignments to a file.
 
     Arguments:
-     - sequences - A list (or iterator) of Alignment objects
-     - handle    - File handle object to write to
+     - alignments - A list (or iterator) of Alignment objects (ideally the
+                   new MultipleSeqAlignment objects), or (if using Biopython
+                   1.54 or later) a single alignment object.
+     - handle    - File handle object to write to, or filename as string
+                   (note older versions of Biopython only took a handle).
      - format    - lower case string describing the file format to write.
 
     You should close the handle after calling this function.
@@ -175,16 +186,22 @@ def write(alignments, handle, format):
     from Bio import SeqIO
 
     #Try and give helpful error messages:
-    if isinstance(handle, basestring):
-        raise TypeError("Need a file handle, not a string (i.e. not a filename)")
     if not isinstance(format, basestring):
         raise TypeError("Need a string for the file format (lower case)")
     if not format:
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
+
     if isinstance(alignments, Alignment):
-        raise TypeError("Need an Alignment list/iterator, not just a single Alignment")
+        #This raised an exception in order version of Biopython
+        alignments = [alignments]
+
+    if isinstance(handle, basestring):
+        handle = open(handle, "w")
+        handle_close = True
+    else:
+        handle_close = False
 
     #Map the file format to a writer class
     if format in _FormatToIterator:
@@ -196,7 +213,8 @@ def write(alignments, handle, format):
         count = 0
         for alignment in alignments:
             if not isinstance(alignment, Alignment):
-                raise TypeError("Expect a list or iterator of Alignment objects.")
+                raise TypeError(\
+                    "Expect a list or iterator of Alignment objects.")
             SeqIO.write(alignment, handle, format)
             count += 1
     elif format in _FormatToIterator or format in SeqIO._FormatToIterator:
@@ -208,11 +226,15 @@ def write(alignments, handle, format):
     assert isinstance(count, int), "Internal error - the underlying %s " \
            "writer should have returned the alignment count, not %s" \
            % (format, repr(count))
+
+    if handle_close:
+        handle.close()
+
     return count
 
 #This is a generator function!
 def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
-    """Uses Bio.SeqIO to create an Alignment iterator (PRIVATE).
+    """Uses Bio.SeqIO to create an MultipleSeqAlignment iterator (PRIVATE).
 
     Arguments:
      - handle    - handle to the file.
@@ -223,8 +245,8 @@ def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
      - seq_count - Optional integer, number of sequences expected in each
                    alignment.  Recommended for fasta format files.
 
-    If count is omitted (default) then all the sequences in
-    the file are combined into a single Alignment.
+    If count is omitted (default) then all the sequences in the file are
+    combined into a single MultipleSeqAlignment.
     """
     from Bio import SeqIO
     assert format in SeqIO._FormatToIterator
@@ -237,7 +259,7 @@ def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
         for record in seq_record_iterator:
             records.append(record)
             if len(records) == seq_count:
-                yield SeqIO.to_alignment(records)
+                yield MultipleSeqAlignment(records, alphabet)
                 records = []
         if len(records) > 0:
             raise ValueError("Check seq_count argument, not enough sequences?")
@@ -246,36 +268,37 @@ def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
         #the SeqRecord objects:
         records = list(SeqIO.parse(handle, format, alphabet))
         if records:
-            yield SeqIO.to_alignment(records)
+            yield MultipleSeqAlignment(records, alphabet)
         else:
             #No alignment found!
             pass
 
 def _force_alphabet(alignment_iterator, alphabet):
-     """Iterate over alignments, over-riding the alphabet (PRIVATE)."""
-     #Assume the alphabet argument has been pre-validated
-     given_base_class = _get_base_alphabet(alphabet).__class__
-     for align in alignment_iterator:
-         if not isinstance(_get_base_alphabet(align._alphabet),
-                           given_base_class):
-             raise ValueError("Specified alphabet %s clashes with "\
-                              "that determined from the file, %s" \
-                              % (repr(alphabet), repr(align._alphabet)))
-         for record in align:
-             if not isinstance(_get_base_alphabet(record.seq.alphabet),
-                               given_base_class):
-                 raise ValueError("Specified alphabet %s clashes with "\
-                                  "that determined from the file, %s" \
-                            % (repr(alphabet), repr(record.seq.alphabet)))
-             record.seq.alphabet = alphabet
-         align._alphabet = alphabet
-         yield align
-    
+    """Iterate over alignments, over-riding the alphabet (PRIVATE)."""
+    #Assume the alphabet argument has been pre-validated
+    given_base_class = _get_base_alphabet(alphabet).__class__
+    for align in alignment_iterator:
+        if not isinstance(_get_base_alphabet(align._alphabet),
+                          given_base_class):
+            raise ValueError("Specified alphabet %s clashes with "\
+                             "that determined from the file, %s" \
+                             % (repr(alphabet), repr(align._alphabet)))
+        for record in align:
+            if not isinstance(_get_base_alphabet(record.seq.alphabet),
+                              given_base_class):
+                raise ValueError("Specified alphabet %s clashes with "\
+                                 "that determined from the file, %s" \
+                           % (repr(alphabet), repr(record.seq.alphabet)))
+            record.seq.alphabet = alphabet
+        align._alphabet = alphabet
+        yield align
+
 def parse(handle, format, seq_count=None, alphabet=None):
-    """Turns a sequence file into an iterator returning Alignment objects.
+    """Iterate over an alignment file as MultipleSeqAlignment objects.
 
     Arguments:
-     - handle    - handle to the file.
+     - handle    - handle to the file, or the filename as a string
+                   (note older verions of Biopython only took a handle).
      - format    - string describing the file format.
      - alphabet  - optional Alphabet object, useful when the sequence type
                    cannot be automatically inferred from the file itself
@@ -288,7 +311,7 @@ def parse(handle, format, seq_count=None, alphabet=None):
     >>> from Bio import AlignIO
     >>> filename = "Emboss/needle.txt"
     >>> format = "emboss"
-    >>> for alignment in AlignIO.parse(open(filename,"rU"), format):
+    >>> for alignment in AlignIO.parse(filename, format):
     ...     print "Alignment of length", alignment.get_alignment_length()
     Alignment of length 124
     Alignment of length 119
@@ -306,9 +329,11 @@ def parse(handle, format, seq_count=None, alphabet=None):
     """
     from Bio import SeqIO
 
-    #Try and give helpful error messages:
     if isinstance(handle, basestring):
-        raise TypeError("Need a file handle, not a string (i.e. not a filename)")
+        handle = open(handle, "rU")
+        #TODO - On Python 2.5+ use with statement to close handle
+
+    #Try and give helpful error messages:
     if not isinstance(format, basestring):
         raise TypeError("Need a string for the file format (lower case)")
     if not format:
@@ -331,7 +356,8 @@ def parse(handle, format, seq_count=None, alphabet=None):
             return iterator_generator(handle, seq_count, alphabet=alphabet)
         except TypeError:
             #It isn't supported.
-            return _force_alphabet(iterator_generator(handle, seq_count), alphabet)
+            return _force_alphabet(iterator_generator(handle, seq_count),
+                                   alphabet)
 
     elif format in SeqIO._FormatToIterator:
         #Exploit the existing SeqIO parser to the dirty work!
@@ -342,10 +368,11 @@ def parse(handle, format, seq_count=None, alphabet=None):
         raise ValueError("Unknown format '%s'" % format)
 
 def read(handle, format, seq_count=None, alphabet=None):
-    """Turns an alignment file into a single Alignment object.
+    """Turns an alignment file into a single MultipleSeqAlignment object.
 
     Arguments:
-     - handle    - handle to the file.
+     - handle    - handle to the file, or the filename as a string
+                   (note older verions of Biopython only took a handle).
      - format    - string describing the file format.
      - alphabet  - optional Alphabet object, useful when the sequence type
                    cannot be automatically inferred from the file itself
@@ -360,7 +387,7 @@ def read(handle, format, seq_count=None, alphabet=None):
     >>> from Bio import AlignIO
     >>> filename = "Clustalw/protein.aln"
     >>> format = "clustal"
-    >>> alignment = AlignIO.read(open(filename, "rU"), format)
+    >>> alignment = AlignIO.read(filename, format)
     >>> print "Alignment of length", alignment.get_alignment_length()
     Alignment of length 411
 
@@ -370,7 +397,7 @@ def read(handle, format, seq_count=None, alphabet=None):
     >>> from Bio import AlignIO
     >>> filename = "Emboss/needle.txt"
     >>> format = "emboss"
-    >>> alignment = AlignIO.read(open(filename, "rU"), format)
+    >>> alignment = AlignIO.read(filename, format)
     Traceback (most recent call last):
         ...
     ValueError: More than one record found in handle
@@ -380,7 +407,7 @@ def read(handle, format, seq_count=None, alphabet=None):
     >>> from Bio import AlignIO
     >>> filename = "Emboss/needle.txt"
     >>> format = "emboss"
-    >>> alignment = AlignIO.parse(open(filename, "rU"), format).next()
+    >>> alignment = AlignIO.parse(filename, format).next()
     >>> print "First alignment has length", alignment.get_alignment_length()
     First alignment has length 124
 
@@ -401,7 +428,7 @@ def read(handle, format, seq_count=None, alphabet=None):
     if second is not None:
         raise ValueError("More than one record found in handle")
     if seq_count:
-        assert len(first.get_all_seqs())==seq_count
+        assert len(first)==seq_count
     return first
 
 def convert(in_file, in_format, out_file, out_format, alphabet=None):
@@ -438,8 +465,10 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
     #after we have opened the file which is a shame.
     count = write(alignments, out_handle, out_format)
     #Must now close any handles we opened
-    if in_close : in_handle.close()
-    if out_close : out_handle.close()
+    if in_close:
+        in_handle.close()
+    if out_close:
+        out_handle.close()
     return count
 
 def _test():
@@ -450,14 +479,22 @@ def _test():
     """
     import doctest
     import os
-    if os.path.isdir(os.path.join("..","..","Tests")):
+    if os.path.isdir(os.path.join("..", "..", "Tests")):
         print "Runing doctests..."
         cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("..","..","Tests"))
+        os.chdir(os.path.join("..", "..", "Tests"))
         doctest.testmod()
         os.chdir(cur_dir)
         del cur_dir
         print "Done"
-        
+    elif os.path.isdir(os.path.join("Tests", "Fasta")):
+        print "Runing doctests..."
+        cur_dir = os.path.abspath(os.curdir)
+        os.chdir(os.path.join("Tests"))
+        doctest.testmod()
+        os.chdir(cur_dir)
+        del cur_dir
+        print "Done"
+
 if __name__ == "__main__":
     _test()
