@@ -7,11 +7,17 @@
 
 
 Classes:
-UndoHandle     File object decorator with support for undo-like operations.
-StringHandle   Wraps a file object around a string.
-SGMLHandle     File object that automatically strips SGML tags from data.
 
-SGMLStripper   Object that strips SGML.
+UndoHandle     File object decorator with support for undo-like operations.
+
+StringHandle   Wraps a file object around a string.
+
+SGMLHandle     File object that automatically strips SGML tags from data
+               (DEPRECATED).
+
+SGMLStripper   Object that strips SGML.  This is now considered OBSOLETE, and
+               is likely to be deprecated in a future release of Biopython,
+               and later removed.
 
 """
 import os
@@ -93,16 +99,23 @@ class UndoHandle:
     def __getattr__(self, attr):
         return getattr(self._handle, attr)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self._handle.close()
+
+
 # I could make this faster by using cStringIO.
 # However, cStringIO (in v1.52) does not implement the
 # readlines method.
 StringHandle = StringIO.StringIO
 
-
-
 class SGMLHandle:
-    """A Python handle that automatically strips SGML tags from data.
+    """A Python handle that automatically strips SGML tags from data (DEPRECATED).
 
+    This module is now considered to be obsolete, and is likely to be
+    deprecated in a future release of Biopython, and later removed.
     """
     def __init__(self, handle):
         """SGMLStripper(handle)
@@ -110,6 +123,12 @@ class SGMLHandle:
         handle is a file handle to SGML-formatted data.
         
         """
+        import warnings
+        warnings.warn("Bio.File.SGMLHandle is deprecated, and will be removed"\
+                      " in a future release of Biopython.  If you want to"\
+                      " continue to use this code, please get in contact via"\
+                      " the mailing lists to avoid its permanent removal from"\
+                      " Biopython.", DeprecationWarning)
         self._handle = handle
         self._stripper = SGMLStripper()
 
