@@ -5,10 +5,14 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-"""
-Access to General Feature Format databases created with Bio::DB:GFF
+"""Access to General Feature Format databases created with BioPerl (DEPRECATED).
 
-based on documentation for Lincoln Stein's Perl Bio::DB::GFF
+This is the "old" Bio.GFF module by Michael Hoffman, which offers access to
+a MySQL database holding GFF data loaded by BioPerl. This code has now been
+deprecated, and will be removed (or at best, relocated) in order to free the
+Bio.GFF namespace for a new GFF parser in Biopython (including GFF3 support).
+
+Based on documentation for Lincoln Stein's Perl Bio::DB::GFF
 
 >>> import os
 >>> import Bio.GFF
@@ -19,8 +23,17 @@ based on documentation for Lincoln Stein's Perl Bio::DB::GFF
 
 """
 
-__version__ = "$Revision: 1.7 $"
-# $Source: /home/repository/biopython/biopython/Bio/GFF/__init__.py,v $
+import warnings
+warnings.warn("The old Bio.GFF module for access to a MySQL GFF database "
+              "created with BioPerl is deprecated, and will be removed (or "
+              "possibly just moved) in a future release of Biopython.  If you "
+              "want to continue to use this code, please get in contact with "
+              "the developers via the mailing lists to avoid its permanent "
+              "removal from Biopython. The plan is to re-use the Bio.GFF "
+              "namespace for a new GFF parsing module.", DeprecationWarning)
+
+__version__ = "$Revision: 1.10 $"
+# $Source: /home/bartek/cvs2bzr/biopython_fastimport/cvs_repo/biopython/Bio/GFF/__init__.py,v $
 
 import exceptions
 import operator
@@ -39,7 +52,6 @@ from Bio.Alphabet import IUPAC
 from Bio import DocSQL
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
-from Bio import Translate
 
 import binning
 import easy
@@ -47,7 +59,6 @@ import GenericTools
 
 
 DEFAULT_ALPHABET = IUPAC.unambiguous_dna
-standard_translator = Translate.unambiguous_dna_by_id[1]
 
 class Segment(object):
     """
@@ -137,9 +148,10 @@ class Feature(object):
     Seq('AATAAA', Alphabet())
     >>> print feature.location()
     NC_001802x.fna:73..78
-    >>> from Bio.SeqIO.FASTA import FastaWriter
-    >>> writer = FastaWriter(sys.stdout)
-    >>> writer.write(feature.record())
+    >>> from Bio import SeqIO
+    >>> record = feature.record()
+    >>> records = [record]
+    >>> SeqIO.write(records, sys.stdout, 'fasta')
     > NC_001802x.fna:73..78
     AATAAA
     >>> feature2 = Feature(location=easy.LocationFromString("NC_001802x.fna:73..78"))
@@ -181,7 +193,7 @@ class Feature(object):
                  location=None,
                  alphabet=DEFAULT_ALPHABET):
         if not isinstance(seqname, (types.StringType, types.NoneType)):
-            raise exceptions.TypeError, "seqname needs to be string"
+            raise exceptions.TypeError("seqname needs to be string")
         self.frame = frame
         self.alphabet = alphabet
         if location is None:
@@ -215,13 +227,13 @@ class Feature(object):
         except TypeError:
             seq.alphabet = self.alphabet
         try:
-            return standard_translator.translate(seq)
+            return seq.translate() #default table
         except AssertionError:
             # if the feature was pickled then we have problems
             import cPickle
             if cPickle.dumps(seq.alphabet) == cPickle.dumps(DEFAULT_ALPHABET):
                 seq.alphabet = DEFAULT_ALPHABET
-                return standard_translator.translate(seq)
+                return seq.translate()
             else:
                 raise
 
