@@ -5,6 +5,7 @@ o Record - All of the information in a GenBank record.
 o Reference - hold reference data for a record.
 o Feature - Hold the information in a Feature Table.
 o Qualifier - Qualifiers on a Feature.
+17-MAR-2009: added support for WGS and WGS_SCAFLD lines.  Ying Huang & Iddo Friedberg
 """
 # local stuff
 import Bio.GenBank
@@ -31,6 +32,10 @@ def _wrapped_genbank(information, indent, wrap_space = 1, split_char = " "):
     """
     info_length = Record.GB_LINE_LENGTH - indent
 
+    if not information:
+        #GenBank files use "." for missing data
+        return ".\n"
+
     if wrap_space:
         info_parts = information.split(split_char)
     else:
@@ -176,6 +181,8 @@ class Record:
         self.sequence = ''
         self.contig = ''
         self.primary=[]
+        self.wgs = ''
+        self.wgs_scafld = []
 
     def __str__(self):
         """Provide a GenBank formatted output option for a Record.
@@ -212,6 +219,8 @@ class Record:
         output += self._base_count_line()
         output += self._origin_line()
         output += self._sequence_line()
+        output += self._wgs_line()
+        output += self._wgs_scafld_line()
         output += self._contig_line()
         output += "//"
         return output
@@ -368,7 +377,8 @@ class Record:
         """Output for ORGANISM line with taxonomy info.
         """
         output = Record.INTERNAL_FORMAT % "ORGANISM"
-        output += "%s\n" % self.organism
+        # Now that species names can be too long, this line can wrap (Bug 2591)
+        output += _wrapped_genbank(self.organism, Record.GB_BASE_INDENT)
         output += " " * Record.GB_BASE_INDENT
         taxonomy_info = ""
         for tax in self.taxonomy:
@@ -462,6 +472,20 @@ class Record:
                 cur_seq_pos += 60
         return output
 
+    def _wgs_line(self):
+            output = ""
+            if self.wgs:
+                    output += Record.BASE_FORMAT % "WGS"
+                    output += self.wgs
+            return output
+
+    def _wgs_scafld_line(self):
+            output = ""
+            if self.wgs_scafld:
+                    output += Record.BASE_FORMAT % "WGS_SCAFLD"
+                    output += self.wgs_scafld
+            return output
+        
     def _contig_line(self):
         """Output for CONTIG location information from RefSeq.
         """
@@ -471,6 +495,7 @@ class Record:
             output += _wrapped_genbank(self.contig,
                                        Record.GB_BASE_INDENT, split_char = ',')
         return output
+        
 
 class Reference:
     """Hold information from a GenBank reference.
