@@ -1,4 +1,4 @@
-# Copyright 2007-2009 by Peter Cock.  All rights reserved.
+# Copyright 2007-2010 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -162,12 +162,24 @@ class InsdcScanner:
                     line = self.handle.readline()
             else:
                 #Build up a list of the lines making up this feature:
-                feature_key = line[2:self.FEATURE_QUALIFIER_INDENT].strip()
-                feature_lines = [line[self.FEATURE_QUALIFIER_INDENT:]]
+                if line[self.FEATURE_QUALIFIER_INDENT]!=" " \
+                and " " in line[self.FEATURE_QUALIFIER_INDENT:]:
+                    #The feature table design enforces a length limit on the feature keys.
+                    #Some third party files (e.g. IGMT's EMBL like files) solve this by
+                    #over indenting the location and qualifiers.
+                    feature_key, line = line[2:].strip().split(None,1)
+                    feature_lines = [line]
+                    import warnings
+                    warnings.warn("Overindented %s feature?" % feature_key)
+                else:
+                    feature_key = line[2:self.FEATURE_QUALIFIER_INDENT].strip()
+                    feature_lines = [line[self.FEATURE_QUALIFIER_INDENT:]]
                 line = self.handle.readline()
                 while line[:self.FEATURE_QUALIFIER_INDENT] == self.FEATURE_QUALIFIER_SPACER \
                 or line.rstrip() == "" : # cope with blank lines in the midst of a feature
-                    feature_lines.append(line[self.FEATURE_QUALIFIER_INDENT:].rstrip())
+                    #Use strip to remove any harmless trailing white space AND and leading
+                    #white space (e.g. out of spec files with too much intentation)
+                    feature_lines.append(line[self.FEATURE_QUALIFIER_INDENT:].strip())
                     line = self.handle.readline()
                 features.append(self.parse_feature(feature_key, feature_lines))
         self.line = line
@@ -512,7 +524,7 @@ class EmblScanner(InsdcScanner):
     FEATURE_END_MARKERS = ["XX"] #XX can also mark the end of many things!
     FEATURE_QUALIFIER_INDENT = 21
     FEATURE_QUALIFIER_SPACER = "FT" + " " * (FEATURE_QUALIFIER_INDENT-2)
-    SEQUENCE_HEADERS=["SQ"] #Remove trailing spaces
+    SEQUENCE_HEADERS=["SQ", "CO"] #Remove trailing spaces
 
     def parse_footer(self):
         """returns a tuple containing a list of any misc strings, and the sequence"""
@@ -637,14 +649,16 @@ class EmblScanner(InsdcScanner):
             'SV' : 'version', # SV line removed in June 2006, now part of ID line
             'DE' : 'definition',
             #'RN' : 'reference_num',
+            #'RC' : reference comment... TODO
             #'RP' : 'reference_bases',
             #'RX' : reference cross reference... DOI or Pubmed
-            'RA' : 'authors',
-            'RT' : 'title',
+            'RG' : 'consrtm', #optional consortium
+            #'RA' : 'authors',
+            #'RT' : 'title',
             'RL' : 'journal',
             'OS' : 'organism',
             'OC' : 'taxonomy',
-            #'DR' : data reference?
+            #'DR' : data reference
             'CC' : 'comment',
             #'XX' : splitter
         }
@@ -672,8 +686,17 @@ class EmblScanner(InsdcScanner):
                 elif line_type == 'RP':
                     # Reformat reference numbers for the GenBank based consumer
                     # e.g. '1-4639675' becomes '(bases 1 to 4639675)'
-                    assert data.count("-")==1
-                    consumer.reference_bases("(bases " + data.replace("-", " to ") + ")")
+                    # and '160-550, 904-1055' becomes '(bases 160 to 550; 904 to 1055)'
+                    parts = [bases.replace("-"," to ").strip() for bases in data.split(",")]
+                    consumer.reference_bases("(bases %s)" % "; ".join(parts))
+                elif line_type == 'RT':
+                    #Remove the enclosing quotes and trailing semi colon.
+                    #Note the title can be split over multiple lines.
+                    if data.startswith('"'):
+                        data = data[1:]
+                    if data.endswith('";'):
+                        data = data[:-2]
+                    consumer.title(data)
                 elif line_type == 'RX':
                     # EMBL support three reference types at the moment:
                     # - PUBMED    PUBMED bibliographic database (NLM)
@@ -706,9 +729,20 @@ class EmblScanner(InsdcScanner):
                     # e.g.
                     # DR   MGI; 98599; Tcrb-V4.
                     #
-                    # TODO - Data reference...
-                    # How should we store the secondary identifier (if present)?  Ignore it?
-                    pass
+                    # TODO - How should we store any secondary identifier?
+                    parts = data.rstrip(".").split(";")
+                    #Turn it into "database_identifier:primary_identifier" to
+                    #mimic the GenBank parser. e.g. "MGI:98599"
+                    consumer.dblink("%s:%s" % (parts[0].strip(),
+                                               parts[1].strip()))
+                elif line_type == 'RA':
+                    # Remove trailing ; at end of authors list
+                    consumer.authors(data.rstrip(";"))
+                elif line_type == 'PR':
+                    # Remove trailing ; at end of the project reference
+                    # In GenBank files this corresponds to the old PROJECT
+                    # line which is being replaced with the DBLINK line.
+                    consumer.project(data.rstrip(";"))
                 elif line_type in consumer_dict:
                     #Its a semi-automatic entry!
                     getattr(consumer, consumer_dict[line_type])(data)
@@ -720,7 +754,26 @@ class EmblScanner(InsdcScanner):
         
     def _feed_misc_lines(self, consumer, lines):
         #TODO - Should we do something with the information on the SQ line(s)?
-        pass
+        lines.append("")
+        line_iter = iter(lines)
+        try:
+            for line in line_iter:
+                if line.startswith("CO   "):
+                    line = line[5:].strip()
+                    contig_location = line
+                    while True:
+                        line = line_iter.next()
+                        if not line:
+                            break
+                        elif line.startswith("CO   "):
+                            #Don't need to preseve the whitespace here.
+                            contig_location += line[5:].strip()
+                        else:
+                            raise ValueError('Expected CO (contig) continuation line, got:\n' + line)
+                    consumer.contig_location(contig_location)
+            return
+        except StopIteration:
+            raise ValueError("Problem in misc lines before sequence")
 
 class GenBankScanner(InsdcScanner):
     """For extracting chunks of information in GenBank files"""
@@ -742,9 +795,7 @@ class GenBankScanner(InsdcScanner):
         while self.line[:self.HEADER_WIDTH].rstrip() in self.SEQUENCE_HEADERS \
         or self.line[:self.HEADER_WIDTH] == " "*self.HEADER_WIDTH \
         or "WGS" == self.line[:3]:
-#            if self.line[-1] == "\n" : self.line = self.line[:-1]
-            self.line = self.line.rstrip(os.linesep)
-            misc_lines.append(self.line)
+            misc_lines.append(self.line.rstrip())
             self.line = self.handle.readline()
             if not self.line:
                 raise ValueError("Premature end of file")
