@@ -779,7 +779,14 @@ class LinearDrawer(AbstractDrawer):
         kwargs['head_length_ratio'] = feature.arrowhead_length
         kwargs['shaft_height_ratio'] = feature.arrowshaft_height
 
+        #Support for clickable links... needs ReportLab 2.4 or later
+        #which added support for links in SVG output.
+        if hasattr(feature, "url") :
+            kwargs["hrefURL"] = feature.url
+            kwargs["hrefTitle"] = feature.name
+
         strand = feature.strand
+        
         # Get sigil for the feature, location dependent on the feature strand
         if strand == 1:
             sigil = method((x0, ctr), (x1, top), color=feature.color,
@@ -880,13 +887,13 @@ class LinearDrawer(AbstractDrawer):
         pos, val = data[0]
         lastfrag, lastx = self.canvas_location(pos)
         lastx += self.x0        # Start xy co-ords
-        lasty = trackheight*(val-minval)/resolution + \
+        lasty = trackheight*(val-midval)/resolution + \
                 self.fragment_lines[lastfrag][0] + ctr
         lastval = val
         # Add a series of lines linking consecutive data points
         for pos, val in data:   
             frag, x = self.canvas_location(pos)
-            x += self.x0        # next xy co-ords            
+            x += self.x0        # next xy co-ords
             y = trackheight*(val-midval)/resolution + \
                 self.fragment_lines[frag][0] + ctr
             if frag == lastfrag:    # Points on the same fragment: draw the line
@@ -904,7 +911,7 @@ class LinearDrawer(AbstractDrawer):
                         self.fragment_lines[frag][0] + ctr
                 line_elements.append(Line(self.x0, tempy, x, y,
                                           strokeColor = graph.poscolor,
-                                          strokeWidth = graph.linewidth))                
+                                          strokeWidth = graph.linewidth))
             lastfrag, lastx, lasty, lastval = frag, x, y, val
             
         return line_elements
