@@ -364,7 +364,7 @@ from Bio.ParserSupport import *
 
 
 class MEMEParser (AbstractParser):
-    """A parser for the text output of the MEME program.
+    """A parser for the text output of the MEME program (OBSOLETE).
     Parses the output into an object of the MEMERecord class.
     
     Methods:
@@ -379,6 +379,8 @@ class MEMEParser (AbstractParser):
     ...    for instance in motif.instances:
     ...        print instance.motif_name, instance.sequence_name, instance.strand, instance.pvalue
     
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
     def __init__ (self):
         """__init__ (self)"""
@@ -393,11 +395,13 @@ class MEMEParser (AbstractParser):
 
 
 class _MEMEScanner:
-    """Scanner for MEME output. 
+    """Scanner for MEME output (OBSOLETE).
     
     Methods:
     feed
-        
+    
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
     
     def feed (self, handle, consumer):
@@ -461,10 +465,13 @@ class _MEMEScanner:
 
 class _MEMEConsumer:
     """
-    Consumer that can receive events from MEME Scanner.
+    Consumer that can receive events from MEME Scanner (OBSOLETE).
     
     This is the Consumer object that should be passed to the 
     MEME Scanner.
+    
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
     
     def __init__ (self):
@@ -543,7 +550,7 @@ class _MEMEConsumer:
 
 class _MASTConsumer:
     """
-    Consumer that can receive events from _MASTScanner.
+    Consumer that can receive events from _MASTScanner (OBSOLETE).
     
     A _MASTConsumer parses lines from a mast text output file.
     The motif match diagrams are parsed using line buffering. 
@@ -551,6 +558,9 @@ class _MASTConsumer:
     required for testing using the Bio.ParserSupport.TaggingConsumer.
     If this variable isn't there, the TaggingConsumer barfs. In
     the _MASTScanner, None is passed in the place of this variable.
+    
+    This class is OBSOLETE; please use the read() function in the module
+    Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
         self.data = MASTRecord()
@@ -773,10 +783,10 @@ class _MASTConsumer:
         pass
     
 
-
 class MASTParser(AbstractParser):
     """
-    Parser for MAST text output. HTML output cannot be parsed, yet. Returns a MASTRecord
+    Parser for MAST text output (OBSOLETE).
+    HTML output cannot be parsed, yet. Returns a MASTRecord
     
     A MASTParser takes a file handle for a MAST text output file and 
     returns a MASTRecord, containing the hits between motifs and 
@@ -795,6 +805,9 @@ class MASTParser(AbstractParser):
     >>>for motif in mast_record.motifs:
     >>>    for instance in motif.instances:
     >>>        print instance.motif_name, instance.sequence_name, instance.strand, instance.pvalue
+    
+    This class is OBSOLETE; please use the read() function in the module
+    Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
         self._consumer = _MASTConsumer()
@@ -808,8 +821,10 @@ class MASTParser(AbstractParser):
 
 class _MASTScanner:
     """
-    Scanner for MAST text output. 
-        
+    Scanner for MAST text output (OBSOLETE).
+    
+    This class is OBSOLETE; please use the read() function in the module
+    Bio.Motif.Parsers.MAST instead.
     """
     def feed (self, handle, consumer):
         if isinstance(handle, File.UndoHandle):
@@ -885,7 +900,7 @@ class _MASTScanner:
 
 
 class MASTRecord:
-    """The class for holding the results from a MAST run.
+    """The class for holding the results from a MAST run (OBSOLETE).
     
     A MASTRecord holds data about matches between motifs and sequences.
     The motifs held by the MASTRecord are objects of the class MEMEMotif.
@@ -899,6 +914,9 @@ class MASTRecord:
     MEMEInstance
     get_motif_by_name (motif_name): returns a MEMEMotif with the given
     name.
+    
+    This class is OBSOLETE; please use the read() function in the module
+    Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
         self.sequences = []
