@@ -3,11 +3,12 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-from Numeric import array, Float0, zeros
 import os
+import numpy
 
 from Bio.SVDSuperimposer import SVDSuperimposer
 from Bio.PDB import *
+from Bio.PDB.PDBExceptions import PDBException
 
 __doc__="""
 Classify protein backbone structure according to Kolodny et al's fragment
@@ -76,7 +77,7 @@ def _read_fragments(size, length, dir="."):
             fid+=1
             continue
         # Add CA coord to Fragment
-        coord=array(map(float, sl[0:3]))
+        coord=numpy.array(map(float, sl[0:3]))
         # XXX= dummy residue name
         f.add_residue("XXX", coord)
     fp.close()
@@ -101,7 +102,7 @@ class Fragment:
         self.counter=0
         self.resname_list=[]
         # CA coordinate matrix
-        self.coords_ca=zeros((length, 3), "d")
+        self.coords_ca=numpy.zeros((length, 3), "d")
         self.fid=fid
 
     def get_resname_list(self):
@@ -134,7 +135,7 @@ class Fragment:
         @type ca_coord: Numeric array with length 3
         """
         if self.counter>=self.length:
-            raise PDBException, "Fragment boundary exceeded."
+            raise PDBException("Fragment boundary exceeded.")
         self.resname_list.append(resname)
         self.coords_ca[self.counter]=ca_coord
         self.counter=self.counter+1
@@ -188,9 +189,9 @@ def _make_fragment_list(pp, length):
             if residue.has_id("CA"):
                 ca=residue["CA"]
             else:
-                raise "CHAINBREAK"
+                raise PDBException("CHAINBREAK")
             if ca.is_disordered():
-                raise "CHAINBREAK"
+                raise PDBException("CHAINBREAK")
             ca_coord=ca.get_coord()
             f.add_residue(resname, ca_coord)
         frag_list.append(f)
@@ -247,7 +248,7 @@ class FragmentMapper:
         elif flength==7:
             self.edge=3
         else:
-            raise PDBException, "Fragment length should be 5 or 7."
+            raise PDBException("Fragment length should be 5 or 7.")
         self.flength=flength
         self.lsize=lsize
         self.reflist=_read_fragments(lsize, flength, fdir)
