@@ -3,14 +3,12 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
-import sys
-
-
 __doc__="""
 Consumer class that builds a Structure object. This is used by 
 the PDBParser and MMCIFparser classes.
 """
 
+import warnings
 
 # My stuff 
 # SMCRA hierarchy
@@ -20,7 +18,7 @@ from Chain import Chain
 from Residue import Residue, DisorderedResidue
 from Atom import Atom, DisorderedAtom 
 
-from PDBExceptions import PDBConstructionException
+from PDBExceptions import PDBConstructionException, PDBConstructionWarning
 
 
 class StructureBuilder:
@@ -33,7 +31,7 @@ class StructureBuilder:
         self.header={}
 
     def _is_completely_disordered(self, residue):
-        "Return 1 if all atoms in the residue have a non blanc altloc."
+        "Return 1 if all atoms in the residue have a non blank altloc."
         atom_list=residue.get_unpacked_list()
         for atom in atom_list:
             altloc=atom.get_altloc()
@@ -82,8 +80,9 @@ class StructureBuilder:
         if self.model.has_id(chain_id):
             self.chain=self.model[chain_id]
             if __debug__:
-                sys.stderr.write("WARNING: Chain %s is discontinuous at line %i.\n" 
-                        % (chain_id, self.line_counter))
+                warnings.warn("WARNING: Chain %s is discontinuous at line %i."
+                              % (chain_id, self.line_counter),
+                              PDBConstructionWarning)
         else:
             self.chain=Chain(chain_id)
             self.model.add(self.chain)
@@ -103,7 +102,7 @@ class StructureBuilder:
         Arguments:
         o resname - string, e.g. "ASN"
         o field - hetero flag, "W" for waters, "H" for 
-            hetero residues, otherwise blanc.
+            hetero residues, otherwise blank.
         o resseq - int, sequence identifier
         o icode - string, insertion code
         """
@@ -117,8 +116,10 @@ class StructureBuilder:
                 # There already is a residue with the id (field, resseq, icode).
                 # This only makes sense in the case of a point mutation.
                 if __debug__:
-                    sys.stderr.write("WARNING: Residue ('%s', %i, '%s') redefined at line %i.\n" 
-                            % (field, resseq, icode, self.line_counter))
+                    warnings.warn("WARNING: Residue ('%s', %i, '%s') "
+                                  "redefined at line %i."
+                                  % (field, resseq, icode, self.line_counter),
+                                  PDBConstructionWarning)
                 duplicate_residue=self.chain[res_id]
                 if duplicate_residue.is_disordered()==2:
                     # The residue in the chain is a DisorderedResidue object.
@@ -137,7 +138,7 @@ class StructureBuilder:
                 else:
                     # Make a new DisorderedResidue object and put all
                     # the Residue objects with the id (field, resseq, icode) in it.
-                    # These residues each should have non-blanc altlocs for all their atoms.
+                    # These residues each should have non-blank altlocs for all their atoms.
                     # If not, the PDB file probably contains an error. 
                     if not self._is_completely_disordered(duplicate_residue):
                         # if this exception is ignored, a residue will be missing
@@ -157,7 +158,8 @@ class StructureBuilder:
         self.chain.add(residue)
         self.residue=residue
 
-    def init_atom(self, name, coord, b_factor, occupancy, altloc, fullname, serial_number=None):
+    def init_atom(self, name, coord, b_factor, occupancy, altloc, fullname,
+                  serial_number=None, element=None):
         """
         Initiate a new Atom object.
 
@@ -168,6 +170,7 @@ class StructureBuilder:
         o occupancy - float
         o altloc - string, alternative location specifier
         o fullname - string, atom name including spaces, e.g. " CA "
+        o element - string, upper case, e.g. "HG" for mercury
         """
         residue=self.residue
         # if residue is None, an exception was generated during
@@ -187,9 +190,13 @@ class StructureBuilder:
                     # name of current atom now includes spaces
                     name=fullname
                     if __debug__:
-                        sys.stderr.write("WARNING: atom names %s and %s differ only in spaces at line %i.\n" 
-                            % (duplicate_fullname, fullname, self.line_counter))
-        atom=self.atom=Atom(name, coord, b_factor, occupancy, altloc, fullname, serial_number)
+                        warnings.warn("WARNING: atom names %s and %s differ "
+                                      "only in spaces at line %i."
+                                      % (duplicate_fullname, fullname,
+                                         self.line_counter),
+                                      PDBConstructionWarning)
+        atom=self.atom=Atom(name, coord, b_factor, occupancy, altloc,
+                            fullname, serial_number, element)
         if altloc!=" ":
             # The atom is disordered
             if residue.has_id(name):
@@ -199,7 +206,7 @@ class StructureBuilder:
                     duplicate_atom.disordered_add(atom)     
                 else:
                     # This is an error in the PDB file:
-                    # a disordered atom is found with a blanc altloc
+                    # a disordered atom is found with a blank altloc
                     # Detach the duplicate atom, and put it in a 
                     # DisorderedAtom object together with the current 
                     # atom.
@@ -210,8 +217,10 @@ class StructureBuilder:
                     disordered_atom.disordered_add(duplicate_atom)
                     residue.flag_disordered()
                     if __debug__:
-                        sys.stderr.write("WARNING: disordered atom found with blanc altloc before line %i.\n"  
-                                % self.line_counter)
+                        warnings.warn("WARNING: disordered atom found "
+                                      "with blank altloc before line %i.\n"
+                                      % self.line_counter,
+                                      PDBConstructionWarning)
             else:
                 # The residue does not contain this disordered atom
                 # so we create a new one.
