@@ -23,7 +23,6 @@ __version__ = 'SPARK-0.6.1'
 
 import re
 import sys
-import string
 
 def _namelist(instance):
     namelist, namedict, classlist = [], {}, [instance.__class__]
@@ -57,10 +56,10 @@ class GenericScanner:
                 rv.append(self.makeRE(name))
 
         rv.append(self.makeRE('t_default'))
-        return string.join(rv, '|')
+        return '|'.join(rv)
 
     def error(self, s, pos):
-        print "Lexical error at position %s" % pos
+        print >>sys.stderr, "Lexical error at position %s" % pos
         raise SystemExit
 
     def tokenize(self, s):
@@ -99,7 +98,7 @@ class GenericParser:
     def preprocess(self, rule, func):   return rule, func
 
     def addRule(self, doc, func):
-        rules = string.split(doc)
+        rules = doc.split()
 
         index = []
         for i in range(len(rules)):
@@ -179,7 +178,7 @@ class GenericParser:
         return None
 
     def error(self, token):
-        print "Syntax error at or near `%s' token" % token
+        print >>sys.stderr, "Syntax error at or near `%s' token" % token
         raise SystemExit
 
     def parse(self, tokens):
@@ -556,9 +555,9 @@ def _dump(tokens, states):
         print 'state', i
         for (lhs, rhs), pos, parent in states[i]:
             print '\t', lhs, '::=',
-            print string.join(rhs[:pos]),
+            print ' '.join(rhs[:pos]),
             print '.',
-            print string.join(rhs[pos:]),
+            print ' '.join(rhs[pos:]),
             print ',', parent
         if i < len(tokens):
             print
