@@ -4,9 +4,27 @@
 # as part of this package.
 
 """
+This module is OBSOLETE.
+Most of the functionality in this module has moved to Bio.ExPASy.Prodoc;
+please see
+
+Bio.ExPASy.Prodoc.read          To read a Prodoc file containing one entry.
+Bio.ExPASy.Prodoc.parse         Iterates over entries in a Prodoc file.
+Bio.ExPASy.Prodoc.Record        Holds Prodoc data.
+Bio.ExPASy.Prodoc.Reference     Holds data from a Prodoc reference.
+
+The other functions and classes in Bio.Prosite.Prodoc (including
+Bio.Prosite.Prodoc.index_file and Bio.Prosite.Prodoc.Dictionary) are
+considered deprecated, and were not moved to Bio.ExPASy.Prodoc. If you use
+this functionality, please contact the Biopython developers at
+biopython-dev@biopython.org to avoid permanent removal of this module from
+Biopython.
+
+
+
+
 This module provides code to work with the prosite.doc file from
-Prosite.
-http://www.expasy.ch/prosite/
+Prosite, available at http://www.expasy.ch/prosite/.
 
 Tested with:
 Release 15.0, July 1998
@@ -28,7 +46,6 @@ RecordParser       Parses a Prodoc record into a Record object.
 
 _Scanner           Scans Prodoc-formatted data.
 _RecordConsumer    Consumes Prodoc data to a Record object.
-Iterator           Iterates over entries in a Prodoc file; DEPRECATED.
 """
 
 from types import *
@@ -93,56 +110,6 @@ class Reference:
         self.authors = ''
         self.citation = ''
 
-class Iterator:
-    """Returns one record at a time from a Prodoc file.
-
-    Methods:
-    next   Return the next record from the stream, or None.
-
-    """
-    def __init__(self, handle, parser=None):
-        """__init__(self, handle, parser=None)
-
-        Create a new iterator.  handle is a file-like object.  parser
-        is an optional Parser object to change the results into another form.
-        If set to None, then the raw contents of the file will be returned.
-
-        """
-        import warnings
-        warnings.warn("Bio.Prosite.Prodoc.Iterator is deprecated; we recommend using the function Bio.Prosite.Prodoc.parse instead. Please contact the Biopython developers at biopython-dev@biopython.org you cannot use Bio.Prosite.Prodoc.parse instead of Bio.Prosite.Prodoc.Iterator.",
-              DeprecationWarning)
-        if type(handle) is not FileType and type(handle) is not InstanceType:
-            raise ValueError("I expected a file handle or file-like object")
-        self._uhandle = File.UndoHandle(handle)
-        self._parser = parser
-
-    def next(self):
-        """next(self) -> object
-
-        Return the next Prodoc record from the file.  If no more records,
-        return None.
-
-        """
-        lines = []
-        while 1:
-            line = self._uhandle.readline()
-            if not line:
-                break
-            lines.append(line)
-            if line[:5] == '{END}':
-                break
-            
-        if not lines:
-            return None
-            
-        data = "".join(lines)
-        if self._parser is not None:
-            return self._parser.parse(File.StringHandle(data))
-        return data
-
-    def __iter__(self):
-        return iter(self.next, None)
-
 class Dictionary:
     """Accesses a Prodoc file using a dictionary interface.
 
@@ -177,89 +144,6 @@ class Dictionary:
     def __getattr__(self, name):
         return getattr(self._index, name)
 
-class ExPASyDictionary:
-    """Access PRODOC at ExPASy using a read-only dictionary interface.
-
-    """
-    def __init__(self, delay=5.0, parser=None):
-        """__init__(self, delay=5.0, parser=None)
-
-        Create a new Dictionary to access PRODOC.  parser is an optional
-        parser (e.g. Prodoc.RecordParser) object to change the results
-        into another form.  If set to None, then the raw contents of the
-        file will be returned.  delay is the number of seconds to wait
-        between each query.
-
-        """
-        import warnings
-        warnings.warn("Bio.Prosite.Prodoc.ExPASyDictionary is deprecated. Please use the function Bio.ExPASy.get_prosite_raw instead.",
-              DeprecationWarning)
-
-        self.delay = delay
-        self.parser = parser
-        self.last_query_time = None
-
-    def __len__(self):
-        raise NotImplementedError("Prodoc contains lots of entries")
-    def clear(self):
-        raise NotImplementedError("This is a read-only dictionary")
-    def __setitem__(self, key, item):
-        raise NotImplementedError("This is a read-only dictionary")
-    def update(self):
-        raise NotImplementedError("This is a read-only dictionary")
-    def copy(self):
-        raise NotImplementedError("You don't need to do this...")
-    def keys(self):
-        raise NotImplementedError("You don't really want to do this...")
-    def items(self):
-        raise NotImplementedError("You don't really want to do this...")
-    def values(self):
-        raise NotImplementedError("You don't really want to do this...")
-    
-    def has_key(self, id):
-        """has_key(self, id) -> bool"""
-        try:
-            self[id]
-        except KeyError:
-            return 0
-        return 1
-
-    def get(self, id, failobj=None):
-        try:
-            return self[id]
-        except KeyError:
-            return failobj
-
-    def __getitem__(self, id):
-        """__getitem__(self, id) -> object
-
-        Return a Prodoc entry.  id is either the id or accession
-        for the entry.  Raises a KeyError if there's an error.
-        
-        """
-        import time
-        from Bio import ExPASy
-        # First, check to see if enough time has passed since my
-        # last query.
-        if self.last_query_time is not None:
-            delay = self.last_query_time + self.delay - time.time()
-            if delay > 0.0:
-                time.sleep(delay)
-        self.last_query_time = time.time()
-
-        try:
-            handle = ExPASy.get_prodoc_entry(id)
-        except IOError:
-            raise KeyError(id)
-        try:
-            handle = File.StringHandle(_extract_record(handle))
-        except ValueError:
-            raise KeyError(id)
-        
-        if self.parser is not None:
-            return self.parser.parse(handle)
-        return handle.read()
-
 class RecordParser(AbstractParser):
     """Parses Prodoc data into a Record object.
 
@@ -445,37 +329,3 @@ def index_file(filename, indexname, rec2key=None):
             raise KeyError("duplicate key %s found" % key)
 
         index[key] = start, length
-
-# This function can be deprecated once Bio.Prosite.Prodoc.ExPASyDictionary
-# is removed.
-def _extract_record(handle):
-    """_extract_record(handle) -> str
-
-    Extract PRODOC data from a web page.  Raises a ValueError if no
-    data was found in the web page.
-
-    """
-    # All the data appears between tags:
-    # <pre width = 80>ID   NIR_SIR; PATTERN.
-    # </PRE>
-    class parser(sgmllib.SGMLParser):
-        def __init__(self):
-            sgmllib.SGMLParser.__init__(self)
-            self._in_pre = 0
-            self.data = []
-        def handle_data(self, data):
-            if self._in_pre:
-                self.data.append(data)
-        def do_br(self, attrs):
-            if self._in_pre:
-                self.data.append('\n')
-        def start_pre(self, attrs):
-            self._in_pre = 1
-        def end_pre(self):
-            self._in_pre = 0
-    p = parser()
-    p.feed(handle.read())
-    data = ''.join(p.data).lstrip()
-    if not data:
-        raise ValueError("No data found in web page.")
-    return data
