@@ -4,7 +4,28 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-"""
+"""Module for working with Prosite files from ExPASy (DEPRECATED).
+
+Most of the functionality in this module has moved to Bio.ExPASy.Prosite;
+please see
+
+Bio.ExPASy.Prosite.read          To read a Prosite file containing one entry.
+Bio.ExPASy.Prosite.parse         Iterates over entries in a Prosite file.
+Bio.ExPASy.Prosite.Record        Holds Prosite data.
+
+For
+scan_sequence_expasy  Scan a sequence for occurrences of Prosite patterns.
+_extract_pattern_hits Extract Prosite patterns from a web page.
+PatternHit            Holds data from a hit against a Prosite pattern.
+please see the new module Bio.ExPASy.ScanProsite.
+
+The other functions and classes in Bio.Prosite (including
+Bio.Prosite.index_file and Bio.Prosite.Dictionary) are considered deprecated,
+and were not moved to Bio.ExPASy.Prosite. If you use this functionality,
+please contact the Biopython developers at biopython-dev@biopython.org to
+avoid permanent removal of this module from Biopython.
+
+
 This module provides code to work with the prosite dat file from
 Prosite.
 http://www.expasy.ch/prosite/
@@ -29,12 +50,20 @@ Record                Holds Prosite data.
 PatternHit            Holds data from a hit against a Prosite pattern.
 Dictionary            Accesses a Prosite file using a dictionary interface.
 RecordParser          Parses a Prosite record into a Record object.
-Iterator              Iterates over entries in a Prosite file; DEPRECATED.
 
 _Scanner              Scans Prosite-formatted data.
 _RecordConsumer       Consumes Prosite data to a Record object.
 
 """
+
+import warnings
+warnings.warn("Bio.Prosite is deprecated, and will be removed in a"\
+              " future release of Biopython. Most of the functionality "
+              " is now provided by Bio.ExPASy.Prosite.  If you want to "
+              " continue to use Bio.Prosite, please get in contact "
+              " via the mailing lists to avoid its permanent removal from"\
+              " Biopython.", DeprecationWarning)
+
 from types import *
 import re
 import sgmllib
@@ -42,7 +71,6 @@ from Bio import File
 from Bio import Index
 from Bio.ParserSupport import *
 
-
 # There is probably a cleaner way to write the read/parse functions
 # if we don't use the "parser = RecordParser(); parser.parse(handle)"
 # approach. Leaving that for the next revision of Bio.Prosite.
@@ -200,67 +228,6 @@ class PatternHit:
                 lines.append("%7s %10s %s" % (' ', range_str, seq))
         return "\n".join(lines)
     
-class Iterator:
-    """Returns one record at a time from a Prosite file.
-
-    Methods:
-    next   Return the next record from the stream, or None.
-
-    """
-    def __init__(self, handle, parser=None):
-        """__init__(self, handle, parser=None)
-
-        Create a new iterator.  handle is a file-like object.  parser
-        is an optional Parser object to change the results into another form.
-        If set to None, then the raw contents of the file will be returned.
-
-        """
-        import warnings
-        warnings.warn("Bio.Prosite.Iterator is deprecated; we recommend using the function Bio.Prosite.parse instead. Please contact the Biopython developers at biopython-dev@biopython.org you cannot use Bio.Prosite.parse instead of Bio.Prosite.Iterator.",
-              DeprecationWarning)
-        if type(handle) is not FileType and type(handle) is not InstanceType:
-            raise ValueError("I expected a file handle or file-like object")
-        self._uhandle = File.UndoHandle(handle)
-        self._parser = parser
-
-    def next(self):
-        """next(self) -> object
-
-        Return the next Prosite record from the file.  If no more records,
-        return None.
-
-        """
-        # Skip the copyright info, if it's the first record.
-        line = self._uhandle.peekline()
-        if line[:2] == 'CC':
-            while 1:
-                line = self._uhandle.readline()
-                if not line:
-                    break
-                if line[:2] == '//':
-                    break
-                if line[:2] != 'CC':
-                    raise ValueError("Oops, where's the copyright?")
-        
-        lines = []
-        while 1:
-            line = self._uhandle.readline()
-            if not line:
-                break
-            lines.append(line)
-            if line[:2] == '//':
-                break
-            
-        if not lines:
-            return None
-            
-        data = "".join(lines)
-        if self._parser is not None:
-            return self._parser.parse(File.StringHandle(data))
-        return data
-    
-    def __iter__(self):
-        return iter(self.next, None)
 
 class Dictionary:
     """Accesses a Prosite file using a dictionary interface.
@@ -296,83 +263,6 @@ class Dictionary:
     def __getattr__(self, name):
         return getattr(self._index, name)
 
-class ExPASyDictionary:
-    """Access PROSITE at ExPASy using a read-only dictionary interface.
-
-    """
-    def __init__(self, delay=5.0, parser=None):
-        """__init__(self, delay=5.0, parser=None)
-
-        Create a new Dictionary to access PROSITE.  parser is an optional
-        parser (e.g. Prosite.RecordParser) object to change the results
-        into another form.  If set to None, then the raw contents of the
-        file will be returned.  delay is the number of seconds to wait
-        between each query.
-
-        """
-        import warnings
-        from Bio.WWW import RequestLimiter
-        warnings.warn("Bio.Prosite.ExPASyDictionary is deprecated. Please use the function Bio.ExPASy.get_prosite_raw instead.",
-              DeprecationWarning)
-        self.parser = parser
-        self.limiter = RequestLimiter(delay)
-
-    def __len__(self):
-        raise NotImplementedError("Prosite contains lots of entries")
-    def clear(self):
-        raise NotImplementedError("This is a read-only dictionary")
-    def __setitem__(self, key, item):
-        raise NotImplementedError("This is a read-only dictionary")
-    def update(self):
-        raise NotImplementedError("This is a read-only dictionary")
-    def copy(self):
-        raise NotImplementedError("You don't need to do this...")
-    def keys(self):
-        raise NotImplementedError("You don't really want to do this...")
-    def items(self):
-        raise NotImplementedError("You don't really want to do this...")
-    def values(self):
-        raise NotImplementedError("You don't really want to do this...")
-    
-    def has_key(self, id):
-        """has_key(self, id) -> bool"""
-        try:
-            self[id]
-        except KeyError:
-            return 0
-        return 1
-
-    def get(self, id, failobj=None):
-        try:
-            return self[id]
-        except KeyError:
-            return failobj
-
-    def __getitem__(self, id):
-        """__getitem__(self, id) -> object
-
-        Return a Prosite entry.  id is either the id or accession
-        for the entry.  Raises a KeyError if there's an error.
-        
-        """
-        from Bio import ExPASy
-        # First, check to see if enough time has passed since my
-        # last query.
-        self.limiter.wait()
-
-        try:
-            handle = ExPASy.get_prosite_entry(id)
-        except IOError:
-            raise KeyError(id)
-        try:
-            handle = File.StringHandle(_extract_record(handle))
-        except ValueError:
-            raise KeyError(id)
-        
-        if self.parser is not None:
-            return self.parser.parse(handle)
-        return handle.read()
-
 class RecordParser(AbstractParser):
     """Parses Prosite data into a Record object.
 
@@ -657,7 +547,7 @@ class _RecordConsumer(AbstractConsumer):
                 # CC   Automatic scaling using reversed database
                 # Throw it away.  (Should I keep it?)
                 continue
-            if col.count("=") == 0 :
+            if col.count("=") == 0:
                 #Missing qualifier!  Can we recover gracefully?
                 #For example, from Bug 2403, in PS50293 have:
                 #CC /AUTHOR=K_Hofmann; N_Hulo
@@ -854,36 +744,3 @@ def index_file(filename, indexname, rec2key=None):
             raise KeyError("duplicate key %s found" % key)
 
         index[key] = start, length
-
-# This function can be deprecated once Bio.Prosite.ExPASyDictionary
-# is removed.
-def _extract_record(handle):
-    """_extract_record(handle) -> str
-
-    Extract PROSITE data from a web page.  Raises a ValueError if no
-    data was found in the web page.
-
-    """
-    # All the data appears between tags:
-    # <pre width = 80>ID   NIR_SIR; PATTERN.
-    # </PRE>
-    class parser(sgmllib.SGMLParser):
-        def __init__(self):
-            sgmllib.SGMLParser.__init__(self)
-            self._in_pre = 0
-            self.data = []
-        def handle_data(self, data):
-            if self._in_pre:
-                self.data.append(data)
-        def do_br(self, attrs):
-            if self._in_pre:
-                self.data.append('\n')
-        def start_pre(self, attrs):
-            self._in_pre = 1
-        def end_pre(self):
-            self._in_pre = 0
-    p = parser()
-    p.feed(handle.read())
-    if not p.data:
-        raise ValueError("No data found in web page.")
-    return "".join(p.data)
