@@ -23,13 +23,18 @@ from Bio.Data import IUPACData, CodonTable
 # {{{ 
 
 def reverse(seq):
-    """Reverse the sequence. Works on string sequences.
+    """Reverse the sequence. Works on string sequences (DEPRECATED).
 
-    e.g.
-    >>> reverse("ACGGT")
+    This function is now deprecated, instead use the string's built in slice
+    method with a step of minus one:
+
+    >>> "ACGGT"[::-1]
     'TGGCA'
-    
     """
+    import warnings
+    warnings.warn("Bio.SeqUtils.reverse() is deprecated, use the string's "
+                  "slice method with a step of minus one instead.",
+                  DeprecationWarning)
     r = list(seq)
     r.reverse()
     return ''.join(r)
@@ -216,13 +221,18 @@ def nt_search(seq, subseq):
 # should be moved to ???
 
 class ProteinX(Alphabet.ProteinAlphabet):
+    """Variant of the extended IUPAC extended protein alphabet (DEPRECATED)."""
     letters = IUPACData.extended_protein_letters + "X"
 
+#Can't add a deprecation warning to the class due to the following line:
 proteinX = ProteinX()
 
 class MissingTable:
     def __init__(self, table):
         self._table = table
+        import warnings
+        warnings.warn("Function Bio.SeqUtils.makeTableX() and related classes ProteinX "
+                      "and MissingTable are deprecated.", DeprecationWarning)
     def get(self, codon, stop_symbol):
         try:
             return self._table.get(codon, stop_symbol)
@@ -463,7 +473,7 @@ def quicker_apply_on_multi_fasta(file, function, *args):
         result = f(*arguments)
         if result:
             results.append('>%s\n%s' % (name, result))
-    handle.close()
+    file.close()
     return results
 
 # }}}
