@@ -246,7 +246,6 @@ class Record:
         return "<%s> %s %s\n%s" % (self.__class__.__name__,
                           self.ID, self.symbol, self.title)
 
-
 def parse(handle):
     while True:
         record = _read(handle)
@@ -336,6 +335,7 @@ UG_INDENT=12
 
 class UnigeneSequenceRecord:
     """Store the information for one SEQUENCE line from a Unigene file
+    (OBSOLETE).
 
     Initialize with the text part of the SEQUENCE line, or nothing.
 
@@ -363,6 +363,9 @@ class UnigeneSequenceRecord:
                  splice junction with any other sequence.  In many
                  cases, they are unspliced transcripts originating
                  from the gene.
+
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
     
     def __init__(self,text=None):
@@ -398,6 +401,7 @@ class UnigeneSequenceRecord:
 
 class UnigeneProtsimRecord:
     """Store the information for one PROTSIM line from a Unigene file
+    (OBSOLETE).
 
     Initialize with the text part of the PROTSIM line, or nothing.
 
@@ -407,6 +411,9 @@ class UnigeneProtsimRecord:
     PROTID=      Sequence ID of protein
     PCT=         Percent alignment
     ALN=         length of aligned region (aa)
+
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
 
     def __init__(self,text=None):
@@ -432,6 +439,7 @@ class UnigeneProtsimRecord:
 
 class UnigeneSTSRecord:
     """Store the information for one STS line from a Unigene file
+    (OBSOLETE).
 
     Initialize with the text part of the STS line, or nothing.
 
@@ -441,6 +449,9 @@ class UnigeneSTSRecord:
     ACC=         GenBank/EMBL/DDBJ accession number of STS [optional field]
     DSEG=        GDB Dsegment number [optional field]
     UNISTS=      identifier in NCBI's UNISTS database
+
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
 
     def __init__(self,text=None):
@@ -464,7 +475,7 @@ class UnigeneSTSRecord:
         
 
 class UnigeneRecord:
-    """Store a Unigene record
+    """Store a Unigene record (OBSOLETE).
 
     Here is what is stored:
     
@@ -486,6 +497,9 @@ class UnigeneRecord:
         self.sts          = []  # STS entries, array of STS entries
                                 # Type UnigeneSTSRecord
         self.txmap        = []  # TXMAP entries, array of TXMap entries
+
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
 
     def __init__(self):
@@ -510,6 +524,8 @@ class UnigeneRecord:
 
 
 class _RecordConsumer(AbstractConsumer):
+    """This class is OBSOLETE; please use the read() function in this module
+    instead."""
 
     def __init__(self):
         self.unigene_record = UnigeneRecord()
@@ -553,7 +569,10 @@ class _RecordConsumer(AbstractConsumer):
     
 
 class _Scanner:
-    """Scans a Unigene Flat File Format file
+    """Scans a Unigene Flat File Format file (OBSOLETE).
+
+    This class is OBSOLETE; please use the read() function in this module
+    instead.
     """
 
     def feed(self, handle, consumer):
@@ -581,6 +600,9 @@ class _Scanner:
 
         
 class RecordParser(AbstractParser):
+    """This class is OBSOLETE; please use the read() function in this module
+    instead."""
+
     def __init__(self):
         self._scanner = _Scanner()
         self._consumer = _RecordConsumer()
@@ -594,6 +616,9 @@ class RecordParser(AbstractParser):
         return self._consumer.unigene_record
 
 class Iterator:
+    """This class is OBSOLETE; please use the parse() function in this module
+    instead."""
+
     def __init__(self, handle, parser=None):
         self._uhandle = File.UndoHandle(handle)
 
