@@ -1,6 +1,6 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
-# Revisions 2007-2008 copyright by Peter Cock.  All rights reserved.
-# Revisions 2008 copyright by Cymon J. Cox.  All rights reserved.
+# Revisions 2007-2009 copyright by Peter Cock.  All rights reserved.
+# Revisions 2008-2009 copyright by Cymon J. Cox.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -10,12 +10,15 @@
 """Implementations of Biopython-like Seq objects on top of BioSQL.
 
 This allows retrival of items stored in a BioSQL database using
-a biopython-like Seq interface.
+a biopython-like SeqRecord and Seq interface.
+
+Note: Currently we do not support recording per-letter-annotations
+(like quality scores) in BioSQL.
 """
 
 from Bio import Alphabet
-from Bio.Seq import Seq
-from Bio.SeqRecord import SeqRecord
+from Bio.Seq import Seq, UnknownSeq
+from Bio.SeqRecord import SeqRecord, _RestrictedDict
 from Bio import SeqFeature
 
 class DBSeq(Seq):  # This implements the biopython Seq interface
@@ -38,7 +41,7 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
         #Note since Python 2.0, __getslice__ is deprecated
         #and __getitem__ is used instead.
         #See http://docs.python.org/ref/sequence-methods.html
-        if isinstance(index, int) :
+        if isinstance(index, int):
             #Return a single letter as a string
             i = index
             if i < 0:
@@ -50,16 +53,16 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
             return self.adaptor.get_subseq_as_string(self.primary_id,
                                                      self.start + i,
                                                      self.start + i + 1)
-        if not isinstance(index, slice) :
+        if not isinstance(index, slice):
             raise ValueError("Unexpected index type")
 
         #Return the (sub)sequence as another DBSeq or Seq object
         #(see the Seq obect's __getitem__ method)
-        if index.start is None :
+        if index.start is None:
             i=0
-        else :
+        else:
             i = index.start
-        if i < 0 :
+        if i < 0:
             #Map to equavilent positive index
             if -i > self._length:
                 raise IndexError(i)
@@ -68,11 +71,11 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
             #Trivial case, should return empty string!
             i = self._length
 
-        if index.stop is None :
+        if index.stop is None:
             j = self._length
-        else :
+        else:
             j = index.stop
-        if j < 0 :
+        if j < 0:
             #Map to equavilent positive index
             if -j > self._length:
                 raise IndexError(j)
@@ -83,11 +86,11 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
         if i >= j:
             #Trivial case, empty string.
             return Seq("", self.alphabet)
-        elif index.step is None or index.step == 1 :
+        elif index.step is None or index.step == 1:
             #Easy case - can return a DBSeq with the start and end adjusted
             return self.__class__(self.primary_id, self.adaptor, self.alphabet,
                                   self.start + i, j - i)
-        else :
+        else:
             #Tricky.  Will have to create a Seq object because of the stride
             full = self.adaptor.get_subseq_as_string(self.primary_id,
                                                      self.start + i,
@@ -115,40 +118,67 @@ class DBSeq(Seq):  # This implements the biopython Seq interface
         #Note - the method name copies that of the MutableSeq object
         return Seq(str(self), self.alphabet)
 
-    def __add__(self, other) :
+    def __add__(self, other):
         #Let the Seq object deal with the alphabet issues etc
         return self.toseq() + other
 
-    def __radd__(self, other) :
+    def __radd__(self, other):
         #Let the Seq object deal with the alphabet issues etc
         return other + self.toseq()
 
 
 def _retrieve_seq(adaptor, primary_id):
+    #The database schema ensures there will be only one matching
+    #row in the table.
+
+    #If an UnknownSeq was recorded, seq will be NULL,
+    #but length will be populated.  This means length(seq)
+    #will return None.
     seqs = adaptor.execute_and_fetchall(
-        "SELECT alphabet, length(seq) FROM biosequence" \
+        "SELECT alphabet, length, length(seq) FROM biosequence" \
         " WHERE bioentry_id = %s", (primary_id,))
-    if seqs:
-        moltype, length = seqs[0]
-        moltype = moltype.lower() #might be upper case in database
-        #We have no way of knowing if these sequences will use IUPAC
-        #alphabets, and we certainly can't assume they are unambiguous!
-        if moltype == "dna":
-            alphabet = Alphabet.generic_dna
-        elif moltype == "rna":
-            alphabet = Alphabet.generic_rna
-        elif moltype == "protein":
-            alphabet = Alphabet.generic_protein
-        elif moltype == "unknown":
-            #This is used in BioSQL/Loader.py and would happen
-            #for any generic or nucleotide alphabets.
-            alphabet = Alphabet.single_letter_alphabet
-        else:
-            raise AssertionError("Unknown moltype: %s" % moltype)
-        seq = DBSeq(primary_id, adaptor, alphabet, 0, int(length))
-        return seq
+    if not seqs : return
+    assert len(seqs) == 1        
+    moltype, given_length, length = seqs[0]
+
+    try:
+        length = int(length)
+        given_length = int(length)
+        assert length == given_length
+        have_seq = True
+    except TypeError:
+        assert length is None
+        seqs = adaptor.execute_and_fetchall(
+            "SELECT alphabet, length, seq FROM biosequence" \
+            " WHERE bioentry_id = %s", (primary_id,))
+        assert len(seqs) == 1
+        moltype, given_length, seq = seqs[0]
+        assert seq is None or seq==""
+        length = int(given_length)
+        have_seq = False
+        del seq
+    del given_length
+        
+    moltype = moltype.lower() #might be upper case in database
+    #We have no way of knowing if these sequences will use IUPAC
+    #alphabets, and we certainly can't assume they are unambiguous!
+    if moltype == "dna":
+        alphabet = Alphabet.generic_dna
+    elif moltype == "rna":
+        alphabet = Alphabet.generic_rna
+    elif moltype == "protein":
+        alphabet = Alphabet.generic_protein
+    elif moltype == "unknown":
+        #This is used in BioSQL/Loader.py and would happen
+        #for any generic or nucleotide alphabets.
+        alphabet = Alphabet.single_letter_alphabet
     else:
-        return None
+        raise AssertionError("Unknown moltype: %s" % moltype)
+
+    if have_seq:
+        return DBSeq(primary_id, adaptor, alphabet, 0, int(length))
+    else:
+        return UnknownSeq(length, alphabet)
 
 def _retrieve_dbxrefs(adaptor, primary_id):
     """Retrieve the database cross references for the sequence."""
@@ -210,6 +240,13 @@ def _retrieve_features(adaptor, primary_id):
                 start -= 1
             if strand == 0:
                 strand = None
+            if strand not in (+1, -1, None):
+                raise ValueError("Invalid strand %s found in database for " \
+                                 "seqfeature_id %s" % (strand, seqfeature_id))
+            if end < start:
+                import warnings
+                warnings.warn("Inverted location start/end (%i and %i) for " \
+                              "seqfeature_id %s" % (start, end, seqfeature_id))
             locations.append( (location_id, start, end, strand) )
         # Get possible remote reference information
         remote_results = adaptor.execute_and_fetchall(
@@ -256,7 +293,7 @@ def _retrieve_features(adaptor, primary_id):
                 #TODO - See Bug 2677 - we don't yet record location_operator,
                 #so for consistency with older versions of Biopython default
                 #to assuming its a join.
-                if not subfeature.location_operator :
+                if not subfeature.location_operator:
                     subfeature.location_operator="join"
                 subfeature.location = SeqFeature.FeatureLocation(start, end)
                 subfeature.strand = strand
@@ -272,7 +309,14 @@ def _retrieve_features(adaptor, primary_id):
             start = locations[0][1]
             end = locations[-1][2]
             feature.location = SeqFeature.FeatureLocation(start, end)
-            feature.strand = feature.sub_features[0].strand
+            # To get the parent strand (as done when parsing GenBank files),
+            # need to consider evil mixed strand examples like this,
+            # join(complement(69611..69724),139856..140087,140625..140650)
+            strands = set(sf.strand for sf in feature.sub_features)
+            if len(strands)==1:
+                feature.strand = feature.sub_features[0].strand
+            else:
+                feature.strand = None # i.e. mixed strands
 
         seq_feature_list.append(feature)
 
@@ -292,7 +336,23 @@ def _retrieve_annotations(adaptor, primary_id, taxon_id):
     annotations.update(_retrieve_qualifier_value(adaptor, primary_id))
     annotations.update(_retrieve_reference(adaptor, primary_id))
     annotations.update(_retrieve_taxon(adaptor, primary_id, taxon_id))
-    return annotations
+    annotations.update(_retrieve_comment(adaptor, primary_id))
+    # Convert values into strings in cases of unicode from the database.
+    # BioSQL could eventually be expanded to be unicode aware.
+    str_anns = {}
+    for key, val in annotations.items():
+        if isinstance(val, list):
+            val = [_make_unicode_into_string(x) for x in val]
+        elif isinstance(val, unicode):
+            val = str(val)
+        str_anns[key] = val
+    return str_anns
+
+def _make_unicode_into_string(text):
+    if isinstance(text, unicode):
+        return str(text)
+    else :
+        return text
 
 def _retrieve_qualifier_value(adaptor, primary_id):
     qvs = adaptor.execute_and_fetchall(
@@ -323,8 +383,10 @@ def _retrieve_reference(adaptor, primary_id):
     references = []
     for start, end, location, title, authors, dbname, accession in refs:
         reference = SeqFeature.Reference()
-        if start: start -= 1
-        reference.location = [SeqFeature.FeatureLocation(start, end)]
+        #If the start/end are missing, reference.location is an empty list
+        if (start is not None) or (end is not None):
+            if start is not None: start -= 1 #python counting
+            reference.location = [SeqFeature.FeatureLocation(start, end)]
         #Don't replace the default "" with None.
         if authors : reference.authors = authors
         if title : reference.title = title
@@ -334,7 +396,10 @@ def _retrieve_reference(adaptor, primary_id):
         elif dbname == 'MEDLINE':
             reference.medline_id = accession
         references.append(reference)
-    return {'references': references}
+    if references:
+        return {'references': references}
+    else:
+        return {}
 
 def _retrieve_taxon(adaptor, primary_id, taxon_id):
     a = {}
@@ -363,19 +428,19 @@ def _retrieve_taxon(adaptor, primary_id, taxon_id):
     #separate SQL query for each entry in the lineage, but it does still
     #appear to be *much* faster.  See Bug 2494. 
     taxonomy = []
-    while taxon_id :
+    while taxon_id:
         name, rank, parent_taxon_id = adaptor.execute_one(
         "SELECT taxon_name.name, taxon.node_rank, taxon.parent_taxon_id" \
         " FROM taxon, taxon_name" \
         " WHERE taxon.taxon_id=taxon_name.taxon_id" \
         " AND taxon_name.name_class='scientific name'" \
         " AND taxon.taxon_id = %s", (taxon_id,))
-        if taxon_id == parent_taxon_id :
+        if taxon_id == parent_taxon_id:
             # If the taxon table has been populated by the BioSQL script
             # load_ncbi_taxonomy.pl this is how top parent nodes are stored.
             # Personally, I would have used a NULL parent_taxon_id here.
             break
-        if rank != "no rank" :
+        if rank != "no rank":
             #For consistency with older versions of Biopython, we are only
             #interested in taxonomy entries with a stated rank.
             #Add this to the start of the lineage list.
@@ -386,6 +451,18 @@ def _retrieve_taxon(adaptor, primary_id, taxon_id):
         a['taxonomy'] = taxonomy
     return a
 
+def _retrieve_comment(adaptor, primary_id):
+    qvs = adaptor.execute_and_fetchall(
+        "SELECT comment_text FROM comment" \
+        " WHERE bioentry_id=%s" \
+        " ORDER BY rank", (primary_id,))
+    comments = [comm[0] for comm in qvs]
+    #Don't want to add an empty list...
+    if comments:
+        return {"comment": comments}
+    else:
+        return {}
+
 class DBSeqRecord(SeqRecord):
     """BioSQL equivalent of the biopython SeqRecord object.
     """
@@ -405,6 +482,15 @@ class DBSeqRecord(SeqRecord):
             self.id = "%s.%s" % (accession, version)
         else:
             self.id = accession
+        #We don't yet record any per-letter-annotations in the
+        #BioSQL database, but we should set this property up
+        #for completeness (and the __str__ method).
+        try:
+            length = len(self.seq)
+        except:
+            #Could be no sequence in the database!
+            length = 0
+        self._per_letter_annotations = _RestrictedDict(length=length)
 
     def __get_seq(self):
         if not hasattr(self, "_seq"):
