@@ -7,35 +7,20 @@ import os
 import sys
 import unittest
 from types import *
+
+import warnings
+warnings.filterwarnings("ignore", category=DeprecationWarning)
 from Bio import Fasta
+warnings.resetwarnings()
 
 from Bio import SeqRecord
 from Bio import Seq
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
 
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
-def testing_suite():
-    """Generate the suite of tests.
-    """
-    test_suite = unittest.TestSuite()
-
-    test_loader = unittest.TestLoader()
-    test_loader.testMethodPrefix = 't_'
-    tests = [RecordTest, ParserTest, IteratorTest]
-    
-    for test in tests:
-        cur_suite = test_loader.loadTestsFromTestCase(test)
-        test_suite.addTest(cur_suite)
-
-    return test_suite
 
 class RecordTest(unittest.TestCase):
-    def t_record_basic(self):
+    def test_record_basic(self):
         """Basic test on Record
         """
         def pbool(b):
@@ -61,7 +46,7 @@ class ParserTest(unittest.TestCase):
         for handle in self.handles:
             handle.close()
 
-    def t_record_parser(self):
+    def test_record_parser(self):
         """Basic operation of the Record Parser.
         """
         parser = Fasta.RecordParser()
@@ -72,7 +57,7 @@ class ParserTest(unittest.TestCase):
             assert len(rec.title) == self.lengths[index][0]
             assert len(rec.sequence) == self.lengths[index][1]
 
-    def t_sequence_parser(self):
+    def test_sequence_parser(self):
         """Basic operation of the Sequence Parser.
         """
         parser = Fasta.SequenceParser()
@@ -85,7 +70,7 @@ class ParserTest(unittest.TestCase):
             assert len(rec.seq) == self.lengths[index][1]
             assert len(rec.description) == self.lengths[index][0]
 
-    def t_sequence_alphabet(self):
+    def test_sequence_alphabet(self):
         """Setting the alphabet for the Sequence Parser.
         """
         parser = Fasta.SequenceParser(alphabet =
@@ -93,7 +78,7 @@ class ParserTest(unittest.TestCase):
         rec = parser.parse(self.handles[0])
         assert rec.seq.alphabet == IUPAC.unambiguous_dna
 
-    def t_sequence_title_convert(self):
+    def test_sequence_title_convert(self):
         """Test title conversion for the Sequence Parser.
         """
         def test_title2ids(title):
@@ -111,7 +96,7 @@ class IteratorTest(unittest.TestCase):
     def tearDown(self):
         self.test_handle.close()
 
-    def t_basic_iterator(self):
+    def test_basic_iterator(self):
         """Ensure the Fasta iterator works returning text.
         """
         i = Fasta.Iterator(self.test_handle)
@@ -128,7 +113,7 @@ class IteratorTest(unittest.TestCase):
         assert i.next() is None
         assert i.next() is None
 
-    def t_new_iterator(self):
+    def test_new_iterator(self):
         """Ensure the Fasta iterator works like a Python 2.2 iterator.
         """
         n = 0
@@ -137,7 +122,7 @@ class IteratorTest(unittest.TestCase):
             n += 1
         assert n == 3
 
-    def t_record_iterator(self):
+    def test_record_iterator(self):
         """Test the iterator with a Record Parser.
         """
         parser = Fasta.RecordParser()
@@ -145,7 +130,7 @@ class IteratorTest(unittest.TestCase):
         for rec in iter(iterator):
             assert isinstance(rec, Fasta.Record)
 
-    def t_sequence_iterator(self):
+    def test_sequence_iterator(self):
         """Test the iterator with a Sequence Parser.
         """
         parser = Fasta.SequenceParser()
@@ -153,7 +138,7 @@ class IteratorTest(unittest.TestCase):
         for rec in iter(iterator):
             assert isinstance(rec, SeqRecord.SeqRecord)
     
-    def t_parsing_comments(self):
+    def test_parsing_comments(self):
         """Parse FASTA files with # style comment lines in them.
         """
         handle = open(os.path.join("Fasta", "f003"))
@@ -163,9 +148,7 @@ class IteratorTest(unittest.TestCase):
             num_recs += 1
         assert num_recs == 2
 
-##The dictionaries code has been deprecated
-#class DictionaryTest(unittest.TestCase):
-#   ...
-        
+
 if __name__ == "__main__":
-    sys.exit(run_tests(sys.argv))
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
