@@ -9,21 +9,10 @@ from Bio import MissingExternalDependencyError
 if not os.environ.has_key("MYSQLPASS"):
     raise MissingExternalDependencyError("Environment is not configured for this test (not important if you do not plan to use Bio.GFF).")
 
+import warnings
+warnings.filterwarnings("ignore", category=DeprecationWarning)
 import Bio.GFF
-
-"""
-#Moved these tests to test_GFF2.py as they don't need the SQL database
-import Bio.GFF.GenericTools
-import Bio.GFF.easy
-
-print "Running Bio.GFF.GenericTools doctests..."
-Bio.GFF.GenericTools._test()
-print "Bio.GFF.GenericTools doctests complete."
-
-print "Running Bio.GFF.easy doctests..."
-Bio.GFF.easy._test()
-print "Bio.GFF.easy doctests complete."
-"""
+warnings.resetwarnings()
 
 print "Running Bio.GFF doctests..."
 Bio.GFF._test()
