@@ -6,7 +6,6 @@
 import commands
 import os
 import shutil
-import sys
 import tempfile
 import unittest
 from Bio.PopGen import FDist
@@ -29,26 +28,6 @@ if not found:
     raise MissingExternalDependencyError(\
         "Install FDist if you want to use Bio.PopGen.FDist.")
 
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
-def testing_suite():
-    """Generate the suite of tests.
-    """
-    print "Running fdist tests, which might take some time, please wait"
-    test_suite = unittest.TestSuite()
-
-    test_loader = unittest.TestLoader()
-    test_loader.testMethodPrefix = 't_'
-    tests = [AppTest]
-    
-    for test in tests:
-        cur_suite = test_loader.loadTestsFromTestCase(test)
-        test_suite.addTest(cur_suite)
-
-    return test_suite
 
 class AppTest(unittest.TestCase):
     """Tests the fdist suite of applications.
@@ -67,18 +46,21 @@ class AppTest(unittest.TestCase):
         self._copyfile('out.cpl', 'out.cpl')
 
     def tearDown(self):
-        for file in os.listdir(self.dirname):
-            os.remove(self.dirname + os.sep + file)
-        os.rmdir(self.dirname)
+        #Not sure how exactly, but its possible the temp directory
+        #may not (still) exist.
+        if os.path.isdir(self.dirname):
+            for file in os.listdir(self.dirname):
+                os.remove(self.dirname + os.sep + file)
+            os.rmdir(self.dirname)
 
-    def t_datacal(self):
+    def test_datacal(self):
         """Test datacal execution.
         """
         fst, samp_size = self.ctrl.run_datacal(data_dir = self.dirname)
         assert (fst - 0.44 < 0.01)
         assert (samp_size == 11)
 
-    def t_fdist(self):
+    def test_fdist(self):
         """Test fdist execution.
         """
         #The number of simulations in real life should be at least 10000,
@@ -88,7 +70,7 @@ class AppTest(unittest.TestCase):
                 data_dir = self.dirname)
         assert(abs(fst - 0.1) < 0.02) #Stochastic result...
 
-    def t_fdist_force_fst(self):
+    def test_fdist_force_fst(self):
         """Test fdist execution approximating Fst.
         """
         #The number of simulations in real life should be at least 10000,
@@ -99,13 +81,13 @@ class AppTest(unittest.TestCase):
                 data_dir = self.dirname)
         assert(abs(fst - 0.09) < 0.05) #Stochastic result...
 
-    def t_cplot(self):
+    def test_cplot(self):
         """Test cplot execution.
         """
         cpl_interval =self.ctrl.run_cplot(data_dir = self.dirname)
         assert(len(cpl_interval) == 8)
 
-    def t_pv(self):
+    def test_pv(self):
         """Test pv execution.
         """
         pv_data = self.ctrl.run_pv(data_dir = self.dirname)
@@ -113,4 +95,6 @@ class AppTest(unittest.TestCase):
 
 
 if __name__ == "__main__":
-    sys.exit(run_tests(sys.argv))
+    print "Running fdist tests, which might take some time, please wait"
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
