@@ -35,77 +35,73 @@ class ScopTests(unittest.TestCase):
 
         cla_out = StringIO()
         scop.write_cla(cla_out)
-        assert cla_out.getvalue() == cla, cla_out.getvalue()
+        self.assertEqual(cla_out.getvalue(), cla)
         
         des_out = StringIO()
         scop.write_des(des_out)
-        assert des_out.getvalue() == des, des_out.getvalue()
+        self.assertEqual(des_out.getvalue(), des)
 
         hie_out = StringIO()
         scop.write_hie(hie_out)
-        assert hie_out.getvalue() == hie, hie_out.getvalue()
+        self.assertEqual(hie_out.getvalue(), hie)
 
         domain = scop.getDomainBySid("d1hbia_")
-        assert domain.sunid == 14996
+        self.assertEqual(domain.sunid, 14996)
 
         domains = scop.getDomains()
-        assert len(domains)==14
-        assert domains[4].sunid == 14988
+        self.assertEqual(len(domains), 14)
+        self.assertEqual(domains[4].sunid, 14988)
 
 
         dom = scop.getNodeBySunid(-111)
-        assert dom == None
+        self.assertEqual(dom, None)
         dom = scop.getDomainBySid("no such domain")
-        assert dom == None
+        self.assertEqual(dom, None)
                 
 
 
     def testSccsOrder(self):
-        assert cmp_sccs("a.1.1.1", "a.1.1.1") == 0
-        assert cmp_sccs("a.1.1.2", "a.1.1.1") == 1
-        assert cmp_sccs("a.1.1.2", "a.1.1.11") == -1
-        assert cmp_sccs("a.1.2.2", "a.1.1.11") == 1
-        assert cmp_sccs("a.1.2.2", "a.5.1.11") == -1         
-        assert cmp_sccs("b.1.2.2", "a.5.1.11") == 1
-        assert cmp_sccs("b.1.2.2", "b.1.2") == 1        
+        self.assertEqual(cmp_sccs("a.1.1.1", "a.1.1.1"), 0)
+        self.assertEqual(cmp_sccs("a.1.1.2", "a.1.1.1"), 1)
+        self.assertEqual(cmp_sccs("a.1.1.2", "a.1.1.11"), -1)
+        self.assertEqual(cmp_sccs("a.1.2.2", "a.1.1.11"), 1)
+        self.assertEqual(cmp_sccs("a.1.2.2", "a.5.1.11"), -1)
+        self.assertEqual(cmp_sccs("b.1.2.2", "a.5.1.11"), 1)
+        self.assertEqual(cmp_sccs("b.1.2.2", "b.1.2"), 1)
 
     def testParseDomain(self):
         s=">d1tpt_1 a.46.2.1 (1-70) Thymidine phosphorylase {Escherichia coli}"
         dom = parse_domain(s)
 
-        assert dom.sid == 'd1tpt_1'
-        assert dom.sccs == 'a.46.2.1'
-        assert dom.residues.pdbid == '1tpt'
-        assert dom.description == 'Thymidine phosphorylase {Escherichia coli}'
+        self.assertEqual(dom.sid, 'd1tpt_1')
+        self.assertEqual(dom.sccs, 'a.46.2.1')
+        self.assertEqual(dom.residues.pdbid, '1tpt')
+        self.assertEqual(dom.description, 'Thymidine phosphorylase {Escherichia coli}')
 
         s2="d1tpt_1 a.46.2.1 (1tpt 1-70) Thymidine phosphorylase {E. coli}"
-        assert s2 == str(parse_domain(s2)), str(parse_domain(s2))
+        self.assertEqual(s2, str(parse_domain(s2)))
 
 
 
         #Genetic domains (See Astral release notes)
         s3="g1cph.1 g.1.1.1 (1cph B:,A:) Insulin {Cow (Bos taurus)}"
-        assert s3 == str(parse_domain(s3)), str(parse_domain(s3))
+        self.assertEqual(s3, str(parse_domain(s3)))
 
         s4="e1cph.1a g.1.1.1 (1cph A:) Insulin {Cow (Bos taurus)}"
-        assert s4 == str(parse_domain(s4))
+        self.assertEqual(s4, str(parse_domain(s4)))
 
         #Raw Astral header
         s5=">e1cph.1a g.1.1.1 (A:) Insulin {Cow (Bos taurus)}"
-        assert s4 ==  str(parse_domain(s5))
+        self.assertEqual(s4,  str(parse_domain(s5)))
 
-        try:
-            dom = parse_domain("Totally wrong")
-            assert False, "Should never get here"
-        except ValueError, e:
-            pass
+        self.assertRaises(ValueError, parse_domain, "Totally wrong")
 
     def testConstructFromDirectory(self):
-         scop = Scop (dir_path="SCOP", version="test")
-         assert isinstance(scop, Scop)
+        scop = Scop (dir_path="SCOP", version="test")
+        self.assertTrue(isinstance(scop, Scop))
          
-         domain = scop.getDomainBySid("d1hbia_")
-         assert domain.sunid == 14996
+        domain = scop.getDomainBySid("d1hbia_")
+        self.assertEqual(domain.sunid, 14996)
          
     def testGetAscendent(self):
         scop = Scop (dir_path="SCOP", version="test")
@@ -113,19 +109,19 @@ class ScopTests(unittest.TestCase):
 
         # get the fold
         fold = domain.getAscendent('cf')
-        assert fold.sunid == 46457
+        self.assertEqual(fold.sunid, 46457)
         
         #get the superfamily
         sf = domain.getAscendent('superfamily')
-        assert sf.sunid == 46458
+        self.assertEqual(sf.sunid, 46458)
 
         # px has no px ascendent
         px = domain.getAscendent('px')
-        assert px == None
+        self.assertEqual(px, None)
 
         # an sf has no px ascendent
         px2 = sf.getAscendent('px')
-        assert px2 == None
+        self.assertEqual(px2, None)
 
 
     def test_get_descendents(self):
@@ -135,22 +131,22 @@ class ScopTests(unittest.TestCase):
 
         # get px descendents
         domains = fold.getDescendents('px')
-        assert len(domains) == 14
+        self.assertEqual(len(domains), 14)
         for d in domains:
-            assert d.type == 'px'
+            self.assertEqual(d.type, 'px')
             
         sfs = fold.getDescendents('superfamily')
-        assert len(sfs) == 1
+        self.assertEqual(len(sfs), 1)
         for d in sfs:
-            assert d.type == 'sf'
+            self.assertEqual(d.type, 'sf')
 
         # cl has no cl descendent
         cl = fold.getDescendents('cl')
-        assert cl == []
+        self.assertEqual(cl, [])
         
         
 
 
-if __name__ == '__main__':
+if __name__=='__main__':
     runner = unittest.TextTestRunner(verbosity = 2)
     unittest.main(testRunner=runner)
