@@ -1,4 +1,4 @@
-# Copyright 2009 by Peter Cock.  All rights reserved.
+# Copyright 2009-2010 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -21,19 +21,20 @@ from StringIO import StringIO
 from Bio.SeqIO.InsdcIO import _insdc_feature_location_string
 
 #Top level function as this makes it easier to use for debugging:
-def write_read(filename, in_format="gb", out_format="gb"):
-    gb_records = list(SeqIO.parse(open(filename),in_format))
-    #Write it out...
-    handle = StringIO()
-    SeqIO.write(gb_records, handle, out_format)
-    handle.seek(0)
-    #Now load it back and check it agrees,
-    gb_records2 = list(SeqIO.parse(handle,out_format))
-    compare_records(gb_records, gb_records2)
+def write_read(filename, in_format="gb", out_formats=["gb", "embl"]):
+    for out_format in out_formats:
+        gb_records = list(SeqIO.parse(open(filename),in_format))
+        #Write it out...
+        handle = StringIO()
+        SeqIO.write(gb_records, handle, out_format)
+        handle.seek(0)
+        #Now load it back and check it agrees,
+        gb_records2 = list(SeqIO.parse(handle,out_format))
+        compare_records(gb_records, gb_records2)
 
-def compare_record(old, new, ignore_name=False):
+def compare_record(old, new, expect_minor_diffs=False):
     #Note the name matching is a bit fuzzy
-    if not ignore_name \
+    if not expect_minor_diffs \
     and old.id != new.id and old.name != new.name \
     and (old.id not in new.id) and (new.id not in old.id) \
     and (old.id.replace(" ","_") != new.id.replace(" ","_")):
@@ -41,27 +42,68 @@ def compare_record(old, new, ignore_name=False):
                          % (old.id, old.name, new.id, new.name))
     if len(old.seq) != len(new.seq):
         raise ValueError("%i vs %i" % (len(old.seq), len(new.seq)))
-    if str(old.seq).upper() != str(new.seq).upper():
+    if isinstance(old.seq, UnknownSeq) \
+    and isinstance(new.seq, UnknownSeq):
+        #Jython didn't like us comparing the string of very long
+        #UnknownSeq object (out of heap memory error)
+        if old.seq._character.upper() != new.seq._character:
+            raise ValueError("%s vs %s" % (repr(old.seq), repr(new.seq)))
+    elif str(old.seq).upper() != str(new.seq).upper():
         if len(old.seq) < 200:
             raise ValueError("'%s' vs '%s'" % (old.seq, new.seq))
         else:
             raise ValueError("'%s...' vs '%s...'" % (old.seq[:100], new.seq[:100]))
     if old.features and new.features:
-        return compare_features(old.features, new.features)
+        if not compare_features(old.features, new.features):
+            return False
     #Just insist on at least one word in common:
     if (old.description or new.description) \
     and not set(old.description.split()).intersection(new.description.split()):
         raise ValueError("%s versus %s" \
                          % (repr(old.description), repr(new.description)))
-    #TODO - check annotation
+    #This only checks common annotation
+    #Would a white list be easier?
+    for key in set(old.annotations.keys()).intersection(new.annotations.keys()):
+        if key in ["data_file_division", "accessions"]:
+            #TODO - These are not yet supported on output, or
+            #have other complications (e.g. different number of accessions
+            #allowed in various file formats)
+            continue
+        if key == "comment":
+            #Ignore whitespace
+            if old.annotations[key].split() != new.annotations[key].split():
+                raise ValueError("Annotation mis-match for comment:\n%s\n%s" \
+                                % (old.annotations[key], new.annotations[key]))
+            continue
+        if key == "references":
+            if expect_minor_diffs:
+                #TODO - Implement EMBL output of references
+                continue
+            assert len(old.annotations[key]) == len(new.annotations[key])
+            for r1, r2 in zip(old.annotations[key], new.annotations[key]):
+                assert r1.title == r2.title
+                assert r1.authors == r2.authors, \
+                       "Old: '%s'\nNew: '%s'" % (r1.authors, r2.authors)
+                assert r1.journal == r2.journal
+                if r1.consrtm and r2.consrtm:
+                    #Not held in EMBL files
+                    assert r1.consrtm == r2.consrtm
+                if r1.medline_id and r2.medline_id:
+                    #Not held in EMBL files
+                    assert r1.medline_id == r2.medline_id
+                assert r1.pubmed_id == r2.pubmed_id
+            continue
+        if repr(old.annotations[key]) != repr(new.annotations[key]):
+            raise ValueError("Annotation mis-match for %s:\n%s\n%s" \
+                             % (key, old.annotations[key], new.annotations[key]))
     return True
 
-def compare_records(old_list, new_list, ignore_name=False):
+def compare_records(old_list, new_list, expect_minor_diffs=False):
     """Check two lists of SeqRecords agree, raises a ValueError if mismatch."""
     if len(old_list) != len(new_list):
         raise ValueError("%i vs %i records" % (len(old_list), len(new_list)))
     for old, new in zip(old_list, new_list):
-        if not compare_record(old,new,ignore_name):
+        if not compare_record(old,new,expect_minor_diffs):
             return False
     return True
 
@@ -165,6 +207,20 @@ class SeqFeatureExtraction(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f), "6..10")
         self.check(s, f, "YWSMK")
 
+    def test_single_letter_dna(self):
+        """Extract feature from DNA (single letter, default strand)"""
+        s = Seq("GATCRYWSMKHBVDN", generic_dna)
+        f = SeqFeature(FeatureLocation(5,6))
+        self.assertEqual(_insdc_feature_location_string(f), "6")
+        self.check(s, f, "Y")
+
+    def test_zero_len_dna(self):
+        """Extract feature from DNA (between location, zero length, default strand)"""
+        s = Seq("GATCRYWSMKHBVDN", generic_dna)
+        f = SeqFeature(FeatureLocation(5,5))
+        self.assertEqual(_insdc_feature_location_string(f), "5^6")
+        self.check(s, f, "")
+
     def test_simple_dna_strand0(self):
         """Extract feature from DNA (simple, strand 0)"""
         s = Seq("GATCRYWSMKHBVDN", generic_dna)
@@ -303,12 +359,16 @@ class FeatureWriting(unittest.TestCase):
     def setUp(self):
         self.record = SeqRecord(Seq("ACGT"*100, generic_dna),
                                 id="Test", name="Test", description="Test")
-    def write_read_check(self):
+    def write_read_check(self, format):
         handle = StringIO()
-        SeqIO.write([self.record], handle, "gb")
+        SeqIO.write([self.record], handle, format)
         handle.seek(0)
-        record2 = SeqIO.read(handle, "gb")
-        return compare_record(self.record, record2)
+        record2 = SeqIO.read(handle, format)
+        compare_record(self.record, record2)
+    
+    def write_read_checks(self, formats=["gb", "embl"]):
+        for f in formats:
+            self.write_read_check(f)
 
     def test_exact(self):
         """Features: write/read simple exact locations."""
@@ -327,7 +387,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "51..60")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
 
     def test_between(self):
         """Features: write/read simple between locations."""
@@ -340,7 +400,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(20^21)")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
 
     def test_join(self):
         """Features: write/read simple join locations."""
@@ -370,7 +430,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(join(311..320,326..340,346..350))")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
 
     def test_fuzzy_join(self):
         """Features: write/read fuzzy join locations."""
@@ -404,8 +464,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(join(>311..320,326..one-of(340,337),346..(350.355)))")
         self.record.features.append(f)
-        self.write_read_check()
-
+        self.write_read_checks()
 
     def test_before(self):
         """Features: write/read simple before locations."""
@@ -439,7 +498,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(56..<60)")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
         
     def test_after(self):
         """Features: write/read simple after locations."""
@@ -473,7 +532,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(56..>60)")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
 
     def test_oneof(self):
         """Features: write/read simple one-of locations."""
@@ -509,7 +568,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(56..one-of(60,63))")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
 
     def test_within(self):
         """Features: write/read simple within locations."""
@@ -545,7 +604,7 @@ class FeatureWriting(unittest.TestCase):
         self.assertEqual(_insdc_feature_location_string(f),
                          "complement(56..(60.65))")
         self.record.features.append(f)
-        self.write_read_check()
+        self.write_read_checks()
         
 class NC_000932(unittest.TestCase):
     #This includes an evil dual strand gene
@@ -601,7 +660,7 @@ class NC_005816(NC_000932):
             return
         gb_record = SeqIO.read(open(self.gb_filename),"genbank")
         embl_record = SeqIO.read(open(self.embl_filename),"embl")
-        return compare_record(gb_record, embl_record, ignore_name=True)
+        return compare_record(gb_record, embl_record, expect_minor_diffs=True)
 
     def test_Translations(self):
         #"""Checking translation of FASTA features (faa vs ffn)."""
@@ -629,7 +688,7 @@ class NC_005816(NC_000932):
         if self.emblname is None:
             return
         embl_record = SeqIO.read(open(self.embl_filename),"embl")
-        compare_record(gb_record, embl_record, ignore_name=True)
+        compare_record(gb_record, embl_record, expect_minor_diffs=True)
 
     def test_Features(self):
         #"""Checking GenBank features sequences vs FASTA ffn file."""
@@ -649,66 +708,98 @@ class NC_005816(NC_000932):
 
 class TestWriteRead(unittest.TestCase):
     """Test can write and read back files."""
-
     def test_NC_000932(self):
         """Write and read back NC_000932.gb"""
-        write_read(os.path.join("GenBank", "NC_000932.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "NC_000932.gb"), "gb")
 
     def test_NC_005816(self):
         """Write and read back NC_005816.gb"""
-        write_read(os.path.join("GenBank", "NC_005816.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "NC_005816.gb"), "gb")
 
     def test_gbvrl1_start(self):
         """Write and read back gbvrl1_start.seq"""
-        write_read(os.path.join("GenBank", "gbvrl1_start.seq"), "gb", "gb")
+        write_read(os.path.join("GenBank", "gbvrl1_start.seq"), "gb")
 
     def test_NT_019265(self):
         """Write and read back NT_019265.gb"""
-        write_read(os.path.join("GenBank", "NT_019265.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "NT_019265.gb"), "gb")
 
     def test_cor6(self):
         """Write and read back cor6_6.gb"""
-        write_read(os.path.join("GenBank", "cor6_6.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "cor6_6.gb"), "gb")
 
     def test_arab1(self):
         """Write and read back arab1.gb"""
-        write_read(os.path.join("GenBank", "arab1.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "arab1.gb"), "gb")
 
     def test_one_of(self):
         """Write and read back of_one.gb"""
-        write_read(os.path.join("GenBank", "one_of.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "one_of.gb"), "gb")
 
     def test_pri1(self):
         """Write and read back pri1.gb"""
-        write_read(os.path.join("GenBank", "pri1.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "pri1.gb"), "gb")
 
     def test_noref(self):
         """Write and read back noref.gb"""
-        write_read(os.path.join("GenBank", "noref.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "noref.gb"), "gb")
 
     def test_origin_line(self):
         """Write and read back origin_line.gb"""
-        write_read(os.path.join("GenBank", "origin_line.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "origin_line.gb"), "gb")
 
     def test_dbsource_wrap(self):
         """Write and read back dbsource_wrap.gb"""
-        write_read(os.path.join("GenBank", "dbsource_wrap.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "dbsource_wrap.gb"), "gb", ["gb"])
+        #Protein so can't convert this to EMBL format
 
     def test_blank_seq(self):
         """Write and read back blank_seq.gb"""
-        write_read(os.path.join("GenBank", "blank_seq.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "blank_seq.gb"), "gb", ["gb"])
+        #Protein so can't convert this to EMBL format
 
     def test_extra_keywords(self):
         """Write and read back extra_keywords.gb"""
-        write_read(os.path.join("GenBank", "extra_keywords.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "extra_keywords.gb"), "gb")
 
     def test_protein_refseq(self):
         """Write and read back protein_refseq.gb"""
-        write_read(os.path.join("GenBank", "protein_refseq.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "protein_refseq.gb"), "gb", ["gb"])
+        #Protein so can't convert this to EMBL format
 
     def test_protein_refseq2(self):
         """Write and read back protein_refseq2.gb"""
-        write_read(os.path.join("GenBank", "protein_refseq2.gb"), "gb", "gb")
+        write_read(os.path.join("GenBank", "protein_refseq2.gb"), "gb", ["gb"])
+        #Protein so can't convert this to EMBL format
+
+    def test_AAA03323(self):
+        """Write and read back AAA03323.embl"""
+        write_read(os.path.join("EMBL", "AAA03323.embl"), "embl")
+
+    def test_AE017046(self):
+        """Write and read back AE017046.embl"""
+        write_read(os.path.join("EMBL", "AE017046.embl"), "embl")
+
+    def test_DD231055_edited(self):
+        """Write and read back DD231055_edited.embl"""
+        write_read(os.path.join("EMBL", "DD231055_edited.embl"), "embl")
+
+    def test_Human_contigs(self):
+        """Write and read back Human_contigs.embl"""
+        write_read(os.path.join("EMBL", "Human_contigs.embl"), "embl")
+
+    def test_SC10H5(self):
+        """Write and read back SC10H5.embl"""
+        write_read(os.path.join("EMBL", "SC10H5.embl"), "embl")
+
+    def test_TRBG361(self):
+        """Write and read back TRBG361.embl"""
+        write_read(os.path.join("EMBL", "TRBG361.embl"), "embl")
+
+    def test_U87107(self):
+        """Write and read back U87107.embl"""
+        write_read(os.path.join("EMBL", "U87107.embl"), "embl")
+
 
 if __name__ == "__main__":
     runner = unittest.TextTestRunner(verbosity = 2)
