@@ -1,12 +1,16 @@
-try :
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+
+try:
     from numpy import array
-except ImportError :
+except ImportError:
     from Bio import MissingExternalDependencyError
     raise MissingExternalDependencyError(\
         "Install NumPy if you want to use Bio.Statistics.lowess.")
 
 from Bio.Statistics.lowess import lowess
-import unittest, sys
+import unittest
 
 class test_lowess(unittest.TestCase):
 
@@ -21,14 +25,6 @@ class test_lowess(unittest.TestCase):
         for precomputed, calculated in zip(ys, output):
             self.assertAlmostEqual(precomputed, calculated, 4)
 
-def testing_suite():
-    s = unittest.makeSuite(test_lowess)
-    return unittest.TestSuite([s])
-
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
 if __name__ == "__main__":
-    sys.exit(run_tests(sys.argv))
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
